require 'incl_combat_abilities'
require 'incl_player_window'
require 'incl_player_names'

charWindowData = {}
local DisplayingTopbarMobileMods = false

function GetEquipSlotUserAction(equipSlot,targetObj,displayName)
	-- no need to send the target obj option if its for the local player	
	local id = equipSlot
	local serverCommand = "equipSlotClicked " .. equipSlot
	if(targetObj ~= nil and targetObj ~= this) then
		id = targetObj.Id.."|"..equipSlot
		serverCommand = "equipSlotClicked " .. equipSlot .. " " .. targetObj.Id
	end
	--DebugMessage("displayName is "..tostring(displayName))
	return {
		ID=id,
		ActionType="EquipSlot",
		DisplayName=displayName,
		Tooltip=displayName,
		Enabled=true,
		ServerCommand=serverCommand,
	}
end

function CanSeeStats(targetObj)
	if(targetObj == this or IsDemiGod(this)) then return true end	
	
	return IsController(this, targetObj)
end

function OnClose(targetObj)
	charWindowData[targetObj] = nil
	UnregisterEventHandler("",EventType.DynamicWindowResponse,"Character"..targetObj.Id)
end

function AddStatEntry(subwindow,curY,targetObj,statName,fullStatName,tooltip,statGainStatus)
	if(statGainStatus) then
		subwindow:AddButton(174,curY,"Toggle"..fullStatName,"",0,0,"","",false,"UpDownLock",GetStatGainState(statGainStatus,fullStatName))
	end

	local labelStr
	if ( targetObj == this ) then
		local base = GetBaseStatValue(targetObj, statName)
		local val = targetObj:GetStatValue(statName)
		local diff = val - base
		if ( base > 0 and ( diff < 0 or diff > 0 ) ) then
			if ( diff > 0 ) then
				labelStr = string.format("(+%d) [STAT:%s]", diff, statName)
			else
				labelStr = string.format("(%d) [STAT:%s]", diff, statName)
			end
		else
			labelStr = string.format("[STAT:%s]", statName)
		end
	else
		labelStr = tostring(math.round(targetObj:GetStatValue(statName), 2))
	end
	local labelX = 170
	if not(statGainStatus) then
		labelX = 180
	end
	subwindow:AddLabel(labelX,curY+2,"[412A08]"..labelStr.."[-]",100,0,18,"right",false,false,"PermianSlabSerif_Dynamic_Bold");
	subwindow:AddButton(0,curY,"","",170,14,tooltip,"",false,"Invisible")
end

function UpdateCharacterWindow(targetObj)
	targetObj = targetObj or this

	if not(charWindowData[targetObj]) then
		charWindowData[targetObj] = { StatTab = "Weapon" }
	end
	charWindowData[targetObj].Open=true

	local dynWindow = DynamicWindow("Character"..targetObj.Id,"",0,0,0,-200,"Character","Center")

	-- The character window has to have 2 subwindows or the content will not appear correctly
	-- so even if you can't see the stats you still have to add an empty subwindow
	local leftSubwindow = Subwindow(0,0)	
	if(CanSeeStats(targetObj)) then		

		-- We are looking at our own character window
		if( targetObj == this ) then
			leftSubwindow:AddImage(10,240,"empty_circle",23,23,"Sliced")
			leftSubwindow:AddImage(10,240,"CraftingCategory_Bashing",23,23,"Sliced")
			if( DisplayingTopbarMobileMods ) then
				leftSubwindow:AddButton(10,240,"Hide","CW",23,23,"Show enchantment statistics.","",false,"Invisible")
			else
				leftSubwindow:AddButton(10,240,"Show","CW",23,23,"Show enchantment statistics.","",false,"Invisible")
			end
		end

		leftSubwindow:AddLabel(10,4,"[412A08][$1677][-]",150,0,20,"",false,false,"PermianSlabSerif_Dynamic_Bold")
		curY = 4
		AddStatEntry(leftSubwindow,curY,targetObj,"Attack","[$3354]")		
		curY = curY + 19
		AddStatEntry(leftSubwindow,curY,targetObj,"AttackSpeed","[$3356]")
		curY = curY + 19
		AddStatEntry(leftSubwindow,curY,targetObj,"Power","[$3352]")
		curY = curY + 19
		AddStatEntry(leftSubwindow,curY,targetObj,"Affinity","[$3353]")
		curY = curY + 19
		AddStatEntry(leftSubwindow,curY,targetObj,"Defense","[$3358]")
		curY = curY + 19
		leftSubwindow:AddLabel(180,curY+2,"[412A08]"..GetVitalityDisplayString(GetCurVitality(targetObj)).."[-]",100,0,18,"right",false,false,"PermianSlabSerif_Dynamic_Bold");
		leftSubwindow:AddButton(0,curY,"","",170,14,"Vitality reflects how rested you are. Low vitality will result in penalties to your maximum health.","",false,"Invisible")		

		leftSubwindow:AddImage(10,120,"Scroll_Divider",174,0,"Sliced")
		leftSubwindow:AddLabel(10,132,"[412A08]Strength\nAgility\nIntelligence\nWisdom\nWill[-]",150,0,20,"",false,false,"PermianSlabSerif_Dynamic_Bold")
		local curY = 132
		local statGainStatus = GetStatGainStatus(targetObj)
		AddStatEntry(leftSubwindow,curY,targetObj,"Str","Strength","[$3360]",statGainStatus)
		curY = curY + 20
		AddStatEntry(leftSubwindow,curY,targetObj,"Agi","Agility","[$3361]",statGainStatus)
		curY = curY + 20
		AddStatEntry(leftSubwindow,curY,targetObj,"Int","Intelligence","[$3362]",statGainStatus)		
		curY = curY + 20
		AddStatEntry(leftSubwindow,curY,targetObj,"Wis","Wisdom","[$3364]",statGainStatus)
		curY = curY + 20
		AddStatEntry(leftSubwindow,curY,targetObj,"Will","Will","[$3365]",statGainStatus)

		if(targetObj == this) then
			leftSubwindow:AddLabel(100,244,"[412A08][STAT:BaseStr+BaseAgi+BaseInt+BaseWis+BaseWill] / "..ServerSettings.Stats.TotalPlayerStatsCap.."[-]",150,0,18,"center",false,false,"PermianSlabSerif_Dynamic_Bold")
			leftSubwindow:AddButton(70,240,"","",60,20,"[$3366]","",false,"Invisible")		
		end

	end		
	dynWindow:AddSubwindow(leftSubwindow)

	local rightSubwindow = Subwindow(0,0)
	
	rightSubwindow:AddButton(10,10,"","Township|"..(TownshipsHelper.GetPlayerTownship(targetObj) or "No Township"),160,40,"","guild",false,"ScrollTitleText")
	
	rightSubwindow:AddButton(10,60,"","Guild|"..(GuildHelpers.GetName(targetObj) or "No Guild"),160,40,"","guild",false,"ScrollTitleText")

	local criminalText = GetCriminalText(targetObj)

	local murderCount = GetMurderCount(targetObj)
	if(murderCount and murderCount > 0) then
		criminalText = criminalText.." (+"..murderCount..")"
	end
	rightSubwindow:AddButton(10,110,"","Criminal Status|"..criminalText,180,40,"","murdercount",false,"ScrollTitleText")
	
	local title = StripColorFromString(targetObj:GetSharedObjectProperty("Title"))
	if(title == "") then
		title = "No Title"
	end
	rightSubwindow:AddButton(10,160,"","Title|"..title,160,40,"","achievement",false,"ScrollTitleText")


	local bio = "Mount Collection"
	rightSubwindow:AddButton(10,210,"MountCollection","Mount Collection|",150,25,"[FF7700]Mount Collection\n[FFFFFF]click here to view your mount collection. Here is where you can set your default mount to use when you use your mount button.","",false,"ScrollTitleText")

	
	dynWindow:AddSubwindow(rightSubwindow)

	dynWindow:AddPortrait(-90,70,180,270,targetObj,"full")	

	-- jewelry slots
	--dynWindow:AddImage(-92,126,"ItemSlot",18,18)
	dynWindow:AddUserAction(-96,126,GetEquipSlotUserAction("Necklace",targetObj,"Necklace"),32,32,"Circle")
	dynWindow:AddUserAction(-96,156,GetEquipSlotUserAction("Ring",targetObj,"Ring"),32,32,"Circle")

	if not(HasHumanAnimations(targetObj)) then
		dynWindow:AddUserAction(-96,186,GetEquipSlotUserAction("Head",targetObj,"Head"),32,32,"Circle")
	end

	-- backpack button state encodes the target id
	if(targetObj:GetEquippedObject("Backpack")) then
		local buttonState = targetObj == this and "" or tostring(targetObj.Id)
		dynWindow:AddButton(62,300,"","",0,0,"","",false,"BackpackButton",buttonState)
	end

	if(targetObj == this) then
		dynWindow:AddButton(-90,246,"ToggleSkills","",46,100,"[$3367]","",false,"SkillBook")
		dynWindow:AddButton(-70,250,"TogglePrestige","",46,95,"[$3368]","",false,"PrestigeBook")
		dynWindow:AddButton(62,312,"ToggleKeyRing","",32,32,"[$3369]","",false,"KeyRing")
	end

	local name = StripColorFromString(targetObj:GetName())
	dynWindow:AddLabel(0,362,name,220,20,22,"center",false,false,"SpectralSC-SemiBold")

	this:OpenDynamicWindow(dynWindow)	

	RegisterEventHandler(EventType.DynamicWindowResponse,"Character"..targetObj.Id, function(...) HandleCharacterWindowResponse(targetObj,...) end)
end

function OpenWriteBioDialog()
	local newWindow = DynamicWindow("WriteBioWindow","Write Bio",400,240,200,200,"Default","TopLeft")

	local currentContents = this:GetObjVar("BioString")

	newWindow:AddTextField(30, 15, 320,120, "contents", currentContents)
	
	newWindow:AddButton(30, 150, "Write", "Write", 150, 0, "", "", false)
	newWindow:AddButton(200, 150, "Cancel", "Cancel", 150, 0, "", "", true)
	--newWindow:AddButton(x, y, "CancelImprovement", "Cancel", 200, 0, "Stop Attempting to Improve the item.", "", false)
	this:OpenDynamicWindow(newWindow)

	RegisterEventHandler(EventType.DynamicWindowResponse,"WriteBioWindow",HandleBioWindowResponse)
end

function OpenMountCollection(category)
	local dynamicWindow = DynamicWindow("MountCollectionWindow","Mount Collection",430,430,-215,-215,"TransparentDraggable","Center")
	local preferedMount = this:GetObjVar("PreferedMount") or {Category="Misc",CategoryKey="SkeletonHorsePvP"}
   local mountCollection = this:GetObjVar("MountCollection")
   dynamicWindow:AddImage(1,1,"HueSwatch",430,430,"Simple","252535",0,0.7)
   dynamicWindow:AddImage(3,3,"HueSwatch",424,424,"Simple","151525",0,0.7)
 
   -- create the header background and text
   dynamicWindow:AddImage(1,1,"HueSwatch",430,34,"Simple","353545")
   dynamicWindow:AddImage(-2,-2,"CraftingItemsFrame",436,40,"Sliced","557575")
   dynamicWindow:AddLabel(205,10,"[959555]Mount Collection - "..category,410,20,20,"center")		

	local curAction = GetPrestigeAbilityUserAction(this, nil,"Skills","MountUp")
	dynamicWindow:AddUserAction(20,40,curAction,46,46)
	dynamicWindow:AddLabel(75,45,"[959555]Use this icon to summon your prefered mount,\nselect a prefered mount from below",340,40,20,"left")
	DrawCloseButton(405,9,dynamicWindow)

   dynamicWindow:AddImage(1,94,"HueSwatch",180,34,"Simple","353545")
   dynamicWindow:AddImage(-2,91,"CraftingItemsFrame",186,40,"Sliced","557575")
   dynamicWindow:AddLabel(90,103,"[959555]Category",180,20,20,"center")	

   dynamicWindow:AddImage(180,110,"VerticalWindowDivider",2,330,"Simple","959555")

   dynamicWindow:AddImage(180,94,"HueSwatch",250,34,"Simple","353545")
   dynamicWindow:AddImage(178,91,"CraftingItemsFrame",256,40,"Sliced","557575")
   dynamicWindow:AddLabel(300,103,"[959555]Mounts Owned",250,20,20,"center")	

	y = 140
	local categoryTextColor = "[959555]"
   for categoryKey,categoryData in pairs(MountCollection) do
   	if(categoryKey == category) then categoryTextColor = "[FF7700]" end
   	dynamicWindow:AddImage(10,y,"HueSwatch",160,30,"Simple","353545")
   	dynamicWindow:AddImage(8,y-4,"CraftingItemsFrame",166,36,"Sliced","557575")
   	dynamicWindow:AddLabel(25,y+6,categoryTextColor..categoryKey,160,20,20,"left")
   	dynamicWindow:AddButton(10,y,"ChangeCategory|"..categoryKey,"",160,20,"[FF7700]"..categoryKey.."\n[959555]click to view mounts in your collection for this category",nil,false,"Invisible")
   	y = y + 36
   	categoryTextColor = "[959555]"
   end


	local scrollWindow = ScrollWindow(200,140,200,210,30)
	local scrollElement = ScrollElement()

	for k,v in pairs(mountCollection[category]) do
		scrollElement:AddLabel(40,0,MountCollection[category][k].Name,160,20,20,"left")
		scrollElement:AddLabel(40,20,MountCollection[category][k].Rarity,160,14,14,"left")
		scrollElement:AddImage(0,0,MountCollection[category][k].Icon,35,35,"Sliced",MountCollection[category][k].IconColor)
		scrollElement:AddButton(0,0,"SelectPreferedMount|"..category..","..k,"",200,40,MountCollection[category][k].Name.."\n[FFFFFF]click to set this mount as your prefered mount",nil,false,"Invisible")
		scrollWindow:Add(scrollElement)
		scrollElement = ScrollElement()
	end
	-- element:AddLabel(24,3,bidInfo.Bidder:GetAttachedUserId(),0,0,18)
	-- element:AddLabel(100,3,bidInfo.Bidder:GetCharacterName() or "N/A",0,0,18,"left")
	-- element:AddLabel(400,3,tostring(bidInfo.BidAmount),1000,0,18,"left")
	-- scrollWindow:Add(element)
	dynamicWindow:AddScrollWindow(scrollWindow)


	this:OpenDynamicWindow(dynamicWindow)

	RegisterEventHandler(EventType.DynamicWindowResponse,"MountCollectionWindow",HandleMountCollectionWindowResponse)
	RegisterEventHandler(EventType.ClientObjectCommand, "dropAction", function (user,sourceId,actionType,actionId,slot) 
		HandleDropAction(user,sourceId,actionType,actionId,slot) 
	end)

end

function HandleMountCollectionWindowResponse(user,buttonId,fields)
	local result = StringSplit(buttonId,"|")
	local button = result[1]
	local buttonArgs = result[2]
	if(button == "ChangeCategory" and buttonArgs) then
		if(buttonArgs == nil) then buttonArgs = "Horses" end
		OpenMountCollection(buttonArgs)
	end

	if(button == "SelectPreferedMount") then
		local argsResult = StringSplit(buttonArgs,",")
		local mainCategory = argsResult[1]
		local subCategory = argsResult[2]
		user:SetObjVar("PreferedMount",{Category=mainCategory,CategoryKey=subCategory})
		user:SystemMessage("you selected " .. MountCollection[mainCategory][subCategory].Name .. " as your prefered mount")
	end

	UnregisterEventHandler("",EventType.ClientObjectCommand,"dropAction")
end

function DrawCloseButton(x,y,dynamicWindow)
	--border background color
	dynamicWindow:AddImage(x,y,"Blank",18,18,"Sliced","555555")
	--background color
	dynamicWindow:AddImage(x+2,y+2,"Blank",14,14,"Sliced","252525")
	--invisible button
	dynamicWindow:AddButton(x,y,"CloseButton|ParentForm","",18,18,"[FF7700]Close Button\n[959555]clicking this will close the mount collection window.","",true,"Invisible")
	--draw the "X" string
	dynamicWindow:AddLabel(x+5,y+3,"[959555]X",18,18,18,"left")
end

function HandleDropAction(user,sourceId,actionType,actionId,slot)
     print(user,sourceId,actionType,actionId,slot)
     if(sourceId == "MountCollectionWindow" and slot ~= nil and user) then

		--local preferedMount = user:GetObjVar("PreferedMount") or {Category="Bears",CategoryKey="GenericBear"}
     	--local mountData =  MountCollection[preferedMount.Category][preferedMount.CategoryKey]
     	local actionData = GetPrestigeAbilityUserAction(user, nil, "Skills", "MountUp")

     	actionData.Slot = tonumber(slot)
        RequestAddUserActionToSlot(user,actionData)
    end
end	




function HandleCharacterWindowResponse(targetObj,user,buttonId)
	if(buttonId == "MountCollection") then
		if(targetObj == this) then
			OpenMountCollection("Horses")
		end
	elseif( buttonId == "Show" ) then
		DisplayingTopbarMobileMods = HelperPlayerTopBar.ShowEnchantmentTrackingButton( this, "Show" )
		UpdateCharacterWindow(targetObj)
	elseif( buttonId == "Hide" ) then
		DisplayingTopbarMobileMods = HelperPlayerTopBar.ShowEnchantmentTrackingButton( this, nil )
		UpdateCharacterWindow(targetObj)
	elseif(buttonId == "WeaponTab") then
		if(charWindowData[targetObj].StatTab ~= "Weapon") then
			charWindowData[targetObj].StatTab = "Weapon"
			UpdateCharacterWindow(targetObj)
		end
	elseif(buttonId == "SpellTab") then
		if(charWindowData[targetObj].StatTab ~= "Spell") then
			charWindowData[targetObj].StatTab = "Spell"
			UpdateCharacterWindow(targetObj)
		end
	elseif(buttonId == "ToggleSkills") then
		if(targetObj == this) then
			ToggleSkillWindow()
		end
	elseif(buttonId == "TogglePrestige") then
		if(targetObj == this) then
			TogglePrestigeWindow()
		end
	elseif(buttonId == "ToggleKeyRing") then
		if(targetObj == this) then
			UpdateKeyRingWindow()
		end
	elseif(buttonId:match("Toggle")) then
		local statName = buttonId:sub(7)
		if(statName and targetObj == this) then
			ToggleStatGainState(this,statName)		
		end
	elseif(buttonId == "") then
		OnClose(targetObj)
	end
end



function HandleBioWindowResponse(user,buttonId,fields)
	if(buttonId == "Write" and fields.contents ~= nil and fields.contents ~= "") then
		-- dont allow colors in guild names
		newBio = StripColorFromString(fields.contents)

		if (string.len(fields.contents) > 150) then
	 		this:SystemMessage("Your bio must be shorter than 150 characters.","info")
	 		return
	 	end

	 	if (fields.contents:match("[^ a-zA-Z0-9.,?!']")) then
		    this:SystemMessage("Your bio can not contain any special characters.","info")
	 		return
		end

		if(ServerSettings.Misc.EnforceBadWordFilter and HasBadWords(fields.contents)) then
	 		this:SystemMessage("You bio can not contain foul language.","info")
	 		return
	 	end

    	this:SetObjVar("BioString",newBio)
    	UpdateCharacterWindow()    	
    end	        

    this:CloseDynamicWindow("WriteBioWindow")
    UnregisterEventHandler("",EventType.DynamicWindowResponse,"WriteBioWindow")
end

function OpenInspectWindow(targetObj)
	if not(charWindowData[targetObj]) then
		UpdateCharacterWindow(targetObj)
	end
end

RegisterEventHandler(EventType.Message,"UpdateCharacterWindow",function ()
	if(charWindowData[this] and charWindowData[this].Open) then
		UpdateCharacterWindow()
	end
end)

RegisterEventHandler(EventType.Message,"OpenCharacterWindow",function ()
	if not(charWindowData[this] and charWindowData[this].Open) then
		UpdateCharacterWindow()
	end
end)

RegisterEventHandler(EventType.ClientUserCommand,"ToggleCharacterWindow",
	function()
		if not(charWindowData[this] and charWindowData[this].Open) then
			UpdateCharacterWindow()
		else
			this:CloseDynamicWindow("Character"..this.Id)
			OnClose(this)			
		end
	end)

RegisterEventHandler(EventType.ItemEquipped, "", function(item)
	if( DisplayingTopbarMobileMods ) then
		CallFunctionDelayed(TimeSpan.FromMilliseconds(40), 
		function()
			DisplayingTopbarMobileMods = HelperPlayerTopBar.ShowEnchantmentTrackingButton( this, "Show" )
		end)
	end
end)