--- master currency table to hold all global information needed by currency items
BaseCurrency = {}

--- these are legal items you can modify randomly with the currency system.
BaseCurrency.LegalCurrencyTargets =
{
	"ArmorType",
	"WeaponType",
	"JewelryType",
	"ShieldType",
}

--- handles the event call back when the player interacts with this object
--@param user - (GameObject) the user interacting with this object
function HandleEventUseObject(user,usedType)
	if(usedType=="Imbue Equipment") then
		user:SystemMessage("select an item you wish to imbue")
		user:RequestClientTargetGameObj(this, "select_target_object")
	end
end

--- handles the targeting call back when the player is asked what object they want to modify
--@param target - (GameObject) the target in which to modify with this currency object
--@param user - (GameObject) the user interacting with this object
function HandleEventSelectTargetObject(target,user)
	for i,v in ipairs(BaseCurrency.LegalCurrencyTargets) do
		-- if target can be modified by this currency
		-- process the currency functions and then
		-- exit out of the function call
		if(target:HasObjVar(v)) then 
			ProcessCurrency(target,user)
			return 
		end
	end
	-- this item can't be modified by this currency in anyway, alert the player
	user:SystemMessage("you can not further modify this item")
end

--- this function is called and is ment to be overriden by most currencies.
--@param target - (GameObject) the target in which to modify with this currency object
--@param user - (GameObject) the user interacting with this object
function ProcessCurrency(target,user)
	-- this function will be empty here as it is ment to differnt for every currency.
end

--- register the event handler for when the player is asked to select a target to modify
RegisterEventHandler(EventType.ClientTargetGameObjResponse, "select_target_object", HandleEventSelectTargetObject)

--- register the event handler for when the player attempts to use the object
RegisterEventHandler(EventType.Message,"UseObject",HandleEventUseObject)

RegisterSingleEventHandler(EventType.ModuleAttached, GetCurrentModule(),
	function()	
        AddUseCase(this,"Imbue Equipment",true)
	end)
