require 'currency.base_currency'

--- TRANSMOGRIFICATION RULES
-- if item is non magical, it will attempt to generate appropriate mods for the targeted item
-- generation of properties on this item will be between 1-2 item properties applied
-- the item will be set to MAGICAL in rarity

--- OVERRIDE BASE FUNCTION CALL ---
--- this function is called and is ment to be overriden by most currencies.
--@param target - (GameObject) the target in which to modify with this currency object
--@param user - (GameObject) the user interacting with this object
function ProcessCurrency(target,user)
	if(target:HasObjVar("WeaponType") or target:HasObjVar("ArmorType")) then
		if(target:HasObjVar("Enchants")) then
			user:SystemMessage("you can only use this on non magical equipment")
			return
		else
			ReworkItemProperties(target)
			SetItemTooltip(target, false)
			if(this:HasObjVar("StackCount")) then
				local stackCount = this:GetObjVar("StackCount")
				stackCount = stackCount - 1
				if(stackCount < 1) then
					this:Destroy()
				else
					this:SendMessage("SetStackCount",stackCount)
				end
			else
				this:Destroy()
			end			
		end
	else
		user:SystemMessage("you cannot transmogrify this item")
		return
	end
end

function ReworkItemProperties(target)
	 local enchants = {}
	 local enchantCount = 0
	 for profileKey,profileValue in pairs(ItemProperties.Profile[target:GetCreationTemplateId()]) do
	 	if(math.random() <= profileValue.Chance) then
	 		local rnd = math.random(1,#profileValue.Enchants)
	 		local modifierValue = math.random(profileValue.Enchants[rnd].Intensity.Min,profileValue.Enchants[rnd].Intensity.Max)
	 		if(profileValue.Enchants[rnd].Intensity.Min < 1 and profileValue.Enchants[rnd].Intensity.Max < 1) then
	 			modifierValue = (math.random(profileValue.Enchants[rnd].Intensity.Min*100,profileValue.Enchants[rnd].Intensity.Max*100))/100
	 		end
	 		table.insert(enchants,
 			{
 				essenceName = profileValue.Enchants[rnd].EssenceKey,
 				essenceModifier = modifierValue,
 				-- profile key is so the alteration gem and chaos gem and exalted gem know what they can still add
 				profileKey = profileKey
 			})
 			enchantCount = enchantCount + 1
	 	end
	 	if(enchantCount > 1) then break end
	 end
	 if(#enchants > 0) then
	 	target:SetObjVar("Enchants",enchants)
	 	target:SetObjVar("MagicLevel",#enchants)
	 end
end