require 'default:globals.engine_callbacks'

-- Return a list of options for interacting with the specified object
-- each option is an array containing the display name and the return id
-- NOTE: empty string is the default id 
function GetObjectInteractionList(user, targetObj)
	local menuItems = {}
	
	local isPermanent = targetObj:IsPermanent()
	if(isPermanent) then
		local sourceId = targetObj:GetSharedObjectProperty("ResourceSourceId")
		if(sourceId ~= nil) then
			if(sourceId:match("Tree")) then
				table.insert(menuItems,"Chop")	
			elseif(sourceId:match("Rock")) then
				table.insert(menuItems,"Mine")
			else
				table.insert(menuItems,"Harvest")
			end
		end
	else
		local useCases = targetObj:GetObjVar("UseCases") or {}
		-- DAB TODO: Should you be able to interact with mobiles while in combat?	
		if ( targetObj:IsMobile() ) then
			local isDead = IsDead(targetObj)
			if ( IsPlayerCharacter(targetObj) ) then
				local groupId = GetGroupId(user)
				if ( targetObj ~= user ) then
					if ( groupId == nil ) then
						local targetGroupId = GetGroupId(targetObj)
						if ( targetGroupId == nil ) then
							table.insert(menuItems, "Invite to Group")
						end
					else
						-- if they are in a group you have to be the leader to invite/kick
						local leader = GetGroupVar(groupId, "Leader")
						if ( leader ~= nil ) then
							if ( leader == user ) then
								local targetGroupId = GetGroupId(targetObj)
								if ( targetGroupId == nil ) then
									table.insert(menuItems, "Invite to Group")
								elseif ( targetGroupId == groupId ) then
									table.insert(menuItems, "Kick from Group")
								end
							end
						end
					end

					if not (IsInFriendList(user, targetObj)) then
						table.insert(menuItems,"Send Friend Request")
					end

					table.insert(menuItems,"Trade")

					-- Are they playing an instrument?; are we not? ROCK ON!
					if( targetObj:HasObjVar("BandID") and user:HasObjVar("BandID") == false and user:HasObjVar("ListeningToBandID") == false ) then
						table.insert(menuItems,"Listen to Music")
					end

					-- Are we listening?
					if( user:HasObjVar("ListeningToBandID") and targetObj:HasObjVar("BandID") and ( user:GetObjVar("ListeningToBandID") == targetObj:GetObjVar("BandID") ) ) then
						table.insert(menuItems,"Stop Listening")
						table.insert(menuItems,"Give Tip")
					end

					-- if they are in a guild and their target is not, let them attempt to invite
					if (
						targetObj:GetObjVar("Guild") == nil
						and
						user:GetObjVar("Guild") ~= nil
					) then
						table.insert(menuItems,"Invite to Guild")
					end
				else
					if ( groupId ~= nil ) then
						table.insert(menuItems, "Group")
					end
				end
				
				--table.insert(menuItems,"Duel")
			else
				if(isDead) then					
					-- else check to see if you can harvest
					--or chop off their head
					if(CanHarvest(targetObj)) then
						table.insert(menuItems,"Harvest")
					end
					if (targetObj:HasObjVar("lootable") or targetObj:HasObjVar("HasPetPack")) then	
						table.insert(menuItems,"Open Pack")
						table.insert(menuItems,"Loot All")
						table.insert(menuItems,"Loot Gold")
					end
					if ( IsPlayerCorpse(targetObj) and IsCriminal(targetObj) ) then
						table.insert(menuItems,"Cut Off Head")
					end
				else
					if (IsController(user,targetObj) and not targetObj:HasObjVar("merchantOwner") ) then						
						if (( IsGod(user) and not TestMortal(user) ) or (IsMount(targetObj) and GetPetSlots(user) > ServerSettings.Pets.MaxSlotsToAllowDismiss)) then
							table.insert(menuItems,"Dismiss")						
						end
					end
				end
			end
			if not(isDead) then
				if(targetObj == user) then
					table.insert(menuItems, "Character")
				elseif ( IsPlayerCharacter(targetObj) or IsPet(targetObj) or (IsHiredMerchant(user,targetObj)) or IsDemiGod(user) ) then
					table.insert(menuItems, "Inspect")
				end
			end
		else
			local equipSlot = GetEquipSlot(targetObj)
			if(CanHarvest(targetObj)) then
				table.insert(menuItems,"Harvest")
			-- DAB TODO: Handle special containers that arent meant to be opened
			-- DAB TODO: Handle containers that you have the key for
			elseif(targetObj:IsContainer() and not targetObj:HasObjVar("FakeContainer")) then
				table.insert(menuItems,"Open")
				if ((not targetObj:IsMobile()) or ((not IsDead(targetObj)) and targetObj:GetEquippedObject("Backpack") ~= nil)) then
					table.insert(menuItems,"Loot All")
				end
				--table.insert(menuItems,"Loot All")
			elseif(equipSlot ~= nil and equipSlot ~= "TempPack" and equipSlot ~= "Bank" and equipSlot ~= "Familiar") then
				if(targetObj:IsEquipped()) then
						table.insert(menuItems,"Unequip")
				elseif (targetObj:TopmostContainer() == user) then
					table.insert(menuItems,"Equip")
				end
			end

			-- this catches lazy scripters that add a use object message but dont add any use cases
			if (#useCases == 0) and (#menuItems == 0) and targetObj:HasEventHandler(EventType.Message,"UseObject") then
				table.insert(menuItems,"Use")
			end

			local objectWeight = targetObj:GetSharedObjectProperty("Weight") or -1
			if(objectWeight ~= -1) then
				table.insert(menuItems,"Pick Up")	
				local topCont = targetObj:TopmostContainer()
				if(topCont ~= user) then
					table.insert(menuItems,"Quick Loot")
				end
			elseif(IsGod(user)) then
				table.insert(menuItems,"God Pick Up")	
			end
		end

		-- EQUIPMENT EDITOR FOR EDITING EQUIPMENT
		if(IsGod(user)) then
			if(targetObj:HasObjVar("ArmorType") or targetObj:HasObjVar("ShieldType") or targetObj:HasObjVar("WeaponType") or targetObj:HasObjVar("JewelryType")) then
    			table.insert(menuItems,"Equipment Editor")
			end
		end

		-- ITEM PROPERTIES WINDOW
		if(targetObj:HasObjVar("ArmorType") or targetObj:HasObjVar("ShieldType") or targetObj:HasObjVar("WeaponType") or targetObj:HasObjVar("JewelryType")) then
    		if(targetObj:HasObjVar("Enchants")) then
    			table.insert(menuItems,"Enchantment Information")
			end
		end
		
		-- SEED OBJECTS
		if(targetObj:HasObjVar("IsSeedObject")) then
			table.insert(menuItems,"Clip Seed Information")
		end
		if (IsGod(user) and ServerSettings.EditMode == false) then
			table.insert(menuItems,"God Info")
			if(targetObj:IsMobile()) then
				table.insert(menuItems,"Edit Mob")

				-- Ranked Arena Master Menu
				if( targetObj:HasObjVar("RankedArenaMaster") ) then
					table.insert(menuItems,"Manage Arena")
				end

			end

			-- Dynamic Spawners
			if( targetObj:HasObjVar("spawnMode") ) then
				table.insert(menuItems,"Manage Dynamic Spawn")
			end

			-- Dynamic Spawners
			if( Gardening.IsPlant(targetObj) ) then
				table.insert(menuItems,"Force Grow")
			end

		end
		if(useCases ~= nil and (#useCases > 0)) then
			for i,useCase in pairs(useCases) do
				if(CanUseCase(user,targetObj,useCase)) then
					table.insert(menuItems,useCase.MenuStr)
				end
			end
		end
	end	

	if (#menuItems == 0) then
		return
	end

	return menuItems
end

