require 'default:globals.helpers.spells'

function CheckSpellCastSuccess(spell, mobileObj, scrollObj)
	if ( SpellData.AllSpells[spell] == nil or SpellData.AllSpells[spell].Skill == nil ) then return false end
	if( scrollObj ) then return true end

	-- mobs always succeed.
	if not( IsPlayerCharacter(mobileObj) ) then return true end
	-- Releasing a primed spell while stunned will result in cast failure.
	if ( HasMobileEffect(mobileObj, "Stun") ) then
		DoFizzle(mobileObj, "Cast failed.")
		return false
	end

	if ( SpellData.AllSpells[spell].Circle == nil ) then
		LuaDebugCallStack("Error: Circle is nil for spell: "..spell)
		return false
	end

	if ( SpellData.AllSpells[spell].Skill == nil ) then
		LuaDebugCallStack("Error: Skill is nil for spell: "..spell)
		return false
	end

	local circle = SpellData.AllSpells[spell].Circle
	local castSkillName = SpellData.AllSpells[spell].Skill

	-- if no scrollObj was passed
	if ( scrollObj == nil ) then
		circle = circle + 2 -- it's two circles easier to cast from scrolls
	end

	local skillDictionary = GetSkillDictionary(mobileObj)

	local skillLevel = GetSkillLevel(mobileObj, castSkillName, skillDictionary) or 0
	local skillGainMinMax = SkillValueMinMax(skillLevel, SpellData.CircleSkills[circle][1], SpellData.CircleSkills[circle][2])
	local success = Success( skillGainMinMax )

	--- when scroll casting, skill check only on a success
	if ( scrollObj == nil or success ) then
		CheckSkillChance(mobileObj, castSkillName, skillLevel, skillGainMinMax)
	end
	
	-- only consume scroll on success
	if ( success and scrollObj ~= nil ) then
		-- consume the scroll, altering success along the way
		local resourceType = scrollObj:GetObjVar("ResourceType")
		-- TODO: MOVE STACKABLE FUNCTIONS TO GLOBALS SO WE CAN JUST ADJUST STACK DIRECTLY ON THE SCROLL OBJ
		success = ( scrollObj:IsValid() and scrollObj:GetObjVar("Spell") == spell 
				and resourceType and ConsumeResourceContainer(mobileObj, resourceType) )
	end

	if not( success ) then
		DoFizzle(mobileObj, "Cast failed.")
	end

	return success
end

function HasSpell(spell, mobileObj, scrollObj)
	-- give mobs all spells
	if not( mobileObj:IsPlayer() ) then return true end
	-- casting from scroll... so..
	if ( scrollObj ~= nil and scrollObj:TopmostContainer() == mobileObj ) then return true end
	-- finally check for a spellbook that has the spell on mobileObj
	local magicKnowledge = mobileObj:GetObjVar("MagicKnowledge") or nil
	if(magicKnowledge == nil) then return false end


	return true
end

---------------------------------------------------------
-- CUSTOM PROJECTILE EFFECT
---------------------------------------------------------
-- Allows me to bypass projectile effects and use this in
-- my mobile effect spells instead.
function DoProjectileSpellEffects(spellCaster,spellTarget,spellName,rootMobileEffect)
	local spellFXName = SpellData.AllSpells[spellName].SpellTravelEffect
	local spellFXArgs = SpellData.AllSpells[spellName].SpellTravelEffectArgs
	local spellHitFX = SpellData.AllSpells[spellName].SpellHitEffect
	local spellHitFXDuration = SpellData.AllSpells[spellName].SpellHitEffectDuration or 2
	local distance = spellTarget:DistanceFrom(spellCaster)

	--errors out if distance is to small 0.25 min seems to work fine and not throw errors
	if(distance < 0.6) then distance = 0.6 end
	
	local overrideRate = 10
	local timer = (distance - .5) / overrideRate
	local spellTime = timer * 1000
	spellCaster:PlayProjectileEffectTo(spellFXName,spellTarget,overrideRate,timer,spellFXArgs)

	CallFunctionDelayed(TimeSpan.FromMilliseconds(spellTime), function()
		-- check for root mobile function calls for projectile spells 
		if(rootMobileEffect.DoDelayedDamage) then
			rootMobileEffect.DoDelayedDamage(rootMobileEffect)
			if(spellHitFX) then spellTarget:PlayEffect(spellHitFX,spellHitFXDuration) end
			return
		end 
	end)	
end