require 'default:globals.static_data.spells'

SpellData.CastTimes =
{
	1, -- circle 1 cast time
	1.25, -- circle 2 cast time
	1.5, -- circle 3 cast time
	1.75, -- circle 4 cast time
	2, -- circle 5 cast time
	3, -- circle 6 cast time
	4, -- circle 7 cast time
	5.5, -- circle 8 cast time
	7, -- circle 9 cast time	
}

SpellData.CircleSkills = 
{
	{-20.1, 20.1},  -- circle -1 (for scrolls)
	{-10.1, 30.1},  -- circle 0 (for scrolls)
	{0.0, 20.1},  -- circle 1
	{0.0, 25.1},  -- circle 2
	{10.1, 50.1}, -- circle 3
	{24.1, 64.1}, -- circle 4
	{38.1, 78.1}, -- circle 5
	{52.1, 92.1}, -- circle 6
	{66.1, 100.1}, -- circle 7
	{80.1, 105.1}, -- circle 8
	{90.1, 110.1}, -- circle 9
}

SpellData.SpellRanges =
{
	Short = 8,
	Medium = 12,
	Long = 16,
}

SpellData.SpellEffectDuration =
{
	BuffSpells = TimeSpan.FromMinutes(10),
	BuffSpellsShort = TimeSpan.FromSeconds(30),
}

---------------------------------------------------------
-- SPELL DURATION RULES
---------------------------------------------------------
-- base effect durations based on spell type
---------------------------------------------------------
SpellData.SpellEffectDuration =
{
	BuffLong = TimeSpan.FromMinutes(5),
	BuffNormal = TimeSpan.FromMinutes(2),
	BuffShort = TimeSpan.FromSeconds(30),
	BuffVeryShort = TimeSpan.FromSeconds(10),
	DebuffLong = TimeSpan.FromMinutes(1),
	DebuffNormal = TimeSpan.FromSeconds(30),
	DebuffShort = TimeSpan.FromSeconds(15),
	Instant = TimeSpan.FromMilliseconds(100),
	Summoning = TimeSpan.FromMinutes(2.5),
	Portal = TimeSpan.FromSeconds(1),
}

---------------------------------------------------------
-- SPELL CAST TIME RULES
---------------------------------------------------------
-- the amount of seconds it takes to cast this spell type
---------------------------------------------------------
SpellData.SpellCastTimes =
{
	BuffLong = 6,
	BuffNormal = 4.5,
	BuffShort = 3,
	BuffInstant = 1,
	DamageHigh = 2.25,
	DamageLow = 1.25,
	DamageNormal = 1.75,
	DebuffLong = 2.5,
	DebuffNormal = 2,
	DebuffShort = 1.25,
	HealHigh = 2.75,
	HealLow = 1.25,
	HealNormal = 2,
	SummoningFast = 1.25,
	SummoningLong = 10,
	SummoningNormal = 5,
	Utility = 2.5,
	Portal = 8,
}

---------------------------------------------------------
-- CASTING SKILL REQUIREMENT RULES
---------------------------------------------------------
-- the min skill to cast and max level to gain skill
---------------------------------------------------------
SpellData.SpellSkillMinMax =
{
	Tier1 = {Min = 0.0,  Max = 20},
	Tier2 = {Min = 15.0, Max = 40},
	Tier3 = {Min = 35.0, Max = 60},
	Tier4 = {Min = 55.0, Max = 80},
	Tier5 = {Min = 75.0, Max = 100},
}

---------------------------------------------------------
-- MANA REQUIREMENT RULES
---------------------------------------------------------
-- the base mana cost for spells
---------------------------------------------------------
SpellData.SpellManaCost =
{
	BuffLong = 22,
	BuffNormal = 14,
	BuffShort = 8,
	DamageHigh = 24,
	DamageLow = 12,
	DamageNormal = 18,
	DebuffLong = 14,
	DebuffNormal = 10,
	DebuffShort = 6,
	HealHigh = 32,
	HealLow = 8,
	HealNormal = 20,
	SummoningHigh = 40,
	SummoningNormal = 20,
	SummoningLow = 8,
	Utility = 16,
	Portal = 50,	
}


SpellData.TextColors =
{
	-- spell casting power word text color
	PowerWords =
	{
		Dark = "[AA55AA]",
		Light = "[AAAA00]",
		Air = "[4477AA]",
		Pyromancy = "[FF9922]",
	},
}

SpellData.AllSpells.Jolt = 
{
	manaCost = 6,
	requireLineOfSight = true,
	effectType = "Projectile",

	PowerWords = "*Jolt*",
	DamageType = "MAGIC",
	MinTravelTime = 0.5,

	Skill = "ElectromancySkill",
	Circle = 1,
	Icon = "Lightning Surge",

	Reagents = 
	{
		"LemonGrass",
	},

	SpellPower = 15,
	SpellRange = SpellData.SpellRanges.Long,
	SpellTravelRate = 60,
	SpellDisplayName = "[CCCC44]Jolt",

	SpellType = "MagicAttackTypeSpell",
	SpellTooltipString = "Conjure a electrical current at your target, jolting them on impact.",

	SpellFireAnim = "spell_fire",

	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",

	SpellFXArgs = "Bone=R_Hand",
	SpellFXDelay = 0.1,
	SpellFXDespawnDelay = 3.0,

	SpellFXName = "LightningBallEffect",
	SpellHitFX = "LightningExplosionEffect",
	
	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
	SpellHitSFX = "event:/magic/fire/magic_fire_fireball_impact",

	CanBeInterrupted = true,
	-- CanClearcast = true,
	-- CanBeDodged = true,
	-- CanBeBlocked = true,
	-- CanBeParried = true,
	-- AttackSpellType = true,
	SpellEnabled = true,
	
	TargetType = "targetMobile",
	TargetRequired = true,

	AttackTypeSpell = true,
}

SpellData.AllSpells.Charged = 
{
	manaCost = 20,
	requireLineOfSight = true,
	effectType = "Projectile",
	MinTravelTime = 0.5,
	PowerWords = "*Jolt*",
	DamageType = "MAGIC",

	Skill = "ElectromancySkill",
	Circle = 2,
	Icon = "Lightning Nova",

	Reagents = 
	{
		"Moss","LemonGrass"
	},

	SpellPower = 10,
	SpellRange = SpellData.SpellRanges.Long,
	SpellTravelRate = 60,
	SpellDisplayName = "[CCCC44]Charged",

	SpellType = "BuffTypeSpell",
	SpellTooltipString = "Charge your target, increasing their movement speed and damage with lightning effects.",

	SpellFireAnim = "spell_fire",

	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",

	SpellFXArgs = "Bone=R_Hand",
	SpellFXDelay = 0.1,
	SpellFXDespawnDelay = 3.0,

	SpellFXName = "LightningBallEffect",
	SpellHitFX = "LightningExplosionEffect",
	
	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
	SpellHitSFX = "event:/magic/fire/magic_fire_fireball_impact",

	CanBeInterrupted = true,
	SpellEnabled = true,
	
	TargetType = "targetMobile",
	TargetRequired = true,
	TargetMobileEffect = "Charged",
}

SpellData.AllSpells.Discharge = 
{
	manaCost = 10,
	requireLineOfSight = true,

	PowerWords = "*Jolt*",
	DamageType = "MAGIC",

	Skill = "ElectromancySkill",
	Circle = 3,
	Icon = "Lightning Orb",

	Reagents = 
	{
		"Moss","LemonGrass"
	},

	SpellPower = 16,
	SpellRange = SpellData.SpellRanges.Long,
	SpellTravelRate = 60,
	SpellDisplayName = "[CCCC44]Discharge",

	SpellType = "BuffTypeSpell",
	SpellTooltipString = "Dischage lightning to surrounding targets, does more damage to single targets around you.",

	SpellFireAnim = "spell_fire",

	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",

	SpellFXArgs = "Bone=R_Hand",
	SpellFXDelay = 0.1,
	SpellFXDespawnDelay = 3.0,

	SpellFXName = "LightningBallEffect",
	SpellHitFX = "LightningExplosionEffect",
	
	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
	SpellHitSFX = "event:/magic/fire/magic_fire_fireball_impact",

	CanBeInterrupted = true,
	SpellEnabled = true,
	
	TargetType = "targetSelf",
	TargetRequired = true,
	TargetMobileEffect = "Discharge",
}

SpellData.AllSpells.Teleport =
{
	PowerWords = "Teleport",
	manaCost = 15,
	DamageType = "MAGIC",
	effectType = "InstantHitSpell",
	SpellPower = 0,
	Icon = "teleport",
	SpellRange = 15,
	TargetType = "targetLocation",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	SpellType = "TeleportSpellType",
	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellPrimeSFX = "event:/magic/air/magic_air_cast_air",
	SpellTooltipString = "Teleports to the target location.",
	requireLineOfSight = true,
	Skill = "ElectromancySkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "cast_teleport",
	SpellDisplayName = "[CCCC44]Teleport",
	SpellReleaseUserScript  = "sp_teleport_effect",
	DistanceDecayRate = -.2,
	DoNotReplaceTarget = true,
	InstantHitSpell = true,
	SpellLaunchSFX = "event:/magic/air/magic_air_teleport",
	ReagentsRequired = false,
	Reagents = {
		"Ginseng",
	},
	Circle = 4,
}

SpellData.AllSpells.LightningBolt = 
{
	Icon = "lightning",
	PowerWords = "Lightning",
	manaCost = 12,
	upkeepCost = 0,
	DamageType = "Energy",
	effectType = "InstantHitSpell",
	SpellPower = 28,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	critEffectTargetScript = "sp_shocked_slow_effect",
	SpellType = "MagicAttackTypeSpell",
	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellFXName = "LightningCloudEffect",
	SpellPrimeSFX = "event:/magic/air/magic_air_cast_air",
	SpellHitSFX = "event:/magic/air/magic_air_lightning_impact",
	SpellSourceSFX = "event:/magic/air/magic_air_lightning",
	SpellTooltipString = "Lightning strikes the target doing instant direct damage.",
	requireLineOfSight = true,
	Skill = "ElectromancySkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "cast_lightning",
	SpellDisplayName = "[CCCC44]Lightning Bolt",
	Reagents = {
		"Moss",
	},
	Circle = 5,
}

SpellData.AllSpells.Electricbolt = 
{
	PowerWords = "Electric Bolt",
	Icon = "Ball Lightning 01",
	MinTravelTime = 0,
	manaCost = 20,
	upkeepCost = 0,
	DamageType = "Energy",
	effectType = "Projectile",
	SpellPower = 38,
	critChance = 0,
	SpellRange = 12,
	SpellTravelRate = 150,
	critEffectTargetScript = "sp_shocked_slow_effect",
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	SpellType = "MagicAttackTypeSpell",
	SpellPrimeFXName = "ConjurePrimeRedEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailRedEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandRedEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandRedEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellFXName = "LightningBallEffect",
	SpellFXArgs = "Bone=R_Hand",
	SpellFXDelay = 0.15,
	SpellFXDespawnDelay = 3,
	SpellHitFX = "ChainLightningExplosionEffect",
	SpellHitSFX = "event:/magic/air/magic_air_energy_bolt_impact",
	SpellPrimeSFX = "event:/magic/air/magic_air_cast_air",
	SpellFlightSFX = "event:/magic/air/magic_air_energy_bolt",
	SpellTooltipString = "Shoots a ball of lightning at your opponent.",
	requireLineOfSight = true,
	Skill = "ElectromancySkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "spell_fire",
	SpellDisplayName = "Energy Bolt",
	CanBeDodged = true,
	CanBeBlocked = true,
	CanBeParried = true,
	AffinityRequired = {
				Air = 2,
		},
	Reagents = {
		"Moss",
	},
	Circle = 7,
}

SpellData.AllSpells.Fireball = 
{
	PowerWords = "Fireball",
	manaCost = 9,
	upkeepCost = 0,
	DamageType = "Fire",
	effectType = "Projectile",
	SpellPower = 19,
	critChance = 1,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	SpellTravelRate = 20,
	critEffectTargetScript = "sp_burn_effect",
	SpellType = "MagicAttackTypeSpell",
	SpellPrimeFXName = "ConjurePrimeRedEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailRedEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandRedEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandRedEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	CanBeDodged = true,
	CanBeBlocked = true,
	CanBeParried = true,
	MinTravelTime = 1,
	SpellFXName = "FireballEffect",
	SpellFXArgs = "Bone=R_Hand",
	SpellFXDelay = 0.15,
	SpellHitFX = "FireballExplosionEffect",
	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
	SpellHitSFX = "event:/magic/fire/magic_fire_fireball_impact",
	SpellFlightSFX = "event:/magic/fire/magic_fire_fireball",
	SpellTooltipString = "Shoots a flaming ball that does fire damage to the target on impact.",
	requireLineOfSight = true,
	Skill = "PyromancySkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "spell_fire",
	SpellDisplayName = "Fireball",
	Reagents = {
		"LemonGrass",
	},
	Circle = 3,
}

-- SpellData.AllSpells.Meteor = 
-- {
-- 	PowerWords = "Meteor",
-- 	manaCost = 40,
-- 	DamageType = "Fire",			
-- 	effectType = "InstantHitSpell",
-- 	SpellPower = 21,
-- 	SpellRange = 12,
-- 	TargetType = "targetLocation",
-- 	TargetRequired = true,
-- 	CanBeInterrupted = true,
-- 	NoMobFrienldyFire = true, -- Stop spell from hurting mobiles with the same "MobileTeamType"
-- 	CanClearcast = true,
-- 	SpellType = "MagicAttackTypeSpell",
-- 	SpellPrimeFXName = "ConjurePrimeRedEffect",
--     SpellPrimeFXArgs = "Bone=Spine2",
--     SpellPrimeFX2Name = "MagicPrimeTrailRedEffect",
--     SpellPrimeFX2Args = "Bone=L_Hand",
--     SpellPrimedFXName = "MagicPrimeHandRedEffect",
--     SpellPrimedFXArgs = "Bone=R_Hand",
--     SpellPrimedFX2Name = "MagicPrimeHandRedEffect",
--     SpellPrimedFX2Args = "Bone=L_Hand",
-- 	SpellTooltipString = "[$3306]",
-- 	requireLineOfSight = true,
-- 	Skill = "PyromancySkill",
-- 	AttackSpellType = true,
-- 	SpellEnabled = true,
-- 	SpellFireAnim = "cast_heal",
-- 	SpellDisplayName = "Meteor",
-- 	SpellReleaseUserScript  = "sp_meteor_effect",
-- 	DoNotReplaceTarget = true,
-- 	InstantHitSpell = true,
-- 	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
-- 	--SpellLaunchSFX = "event:/magic/earth/magic_earth_cast_meteor",
-- 	SpellLaunchSFX = "FallingMeteor",
-- 	critEffectTargetScript = "sp_burn_effect",
-- 	Icon = "Hellstorm",
-- 	Reagents = {
-- 		"LemonGrass",
-- 	},
-- 	Circle = 7,
-- }

SpellData.AllSpells.Ruin = 
{
	PowerWords = "Ruin",
	Icon = "Arcane Arrow",
	manaCost = 6,
	upkeepCost = 0,
	DamageType = "Energy",
	effectType = "InstantHitSpell",
	SpellPower = 17,
	critChance = 10,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	critEffectTargetScript = "sp_shocked_slow_effect",
	SpellType = "MagicAttackTypeSpell",
	--SpellFXName = "LightningCloudEffect",
	SpellPrimeSFX = "event:/magic/void/magic_void_cast_void",
	SpellHitSFX = "event:/magic/fire/magic_fire_fireball_impact",
	SpellPrimeFXName = "ConjurePrimeRedEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailRedEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandRedEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandRedEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellHitFX = "VoidExplosionEffect",
	SpellTooltipString = "Harm your opponent, dealing increased damage when closer to your target.",
	requireLineOfSight = true,
	Skill = "ConjurationSkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "spell_fire",
	SpellDisplayName = "Ruin",
	Reagents = {
		"Moss",
	},
	Circle = 2,
}

SpellData.AllSpells.Bombardment = 
{
	PowerWords = "Bombardment",
	Icon = "Meteor 01",
	manaCost = 20,
	upkeepCost = 0,
	DamageType = "MAGIC",
	SpellPower = 38,
	critChance = 4,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	IndirectProjectile = true,
	SpellTravelRate = .2,
	MinTravelTime = 3.5,
	--critEffectTargetScript = "sp_shocked_slow_effect",
	SpellType = "MagicAttackTypeSpell",
	--SpellFXName = "LightningCloudEffect",
	--SpellLaunchFX = "RockLaunchEffect",
	SpellPrimeFXName = "ConjurePrimeRedEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailRedEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandRedEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandRedEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
	SpellHitSFX = "event:/magic/air/magic_air_lightning",
	SpellHitFX = "RedCoreImpactWaveEffect",
	SpellTooltipString = "Launch a boulder towards the target that impacts after a delay.",
	requireLineOfSight = true,
	Skill = "ConjurationSkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "cast_earth",
	SpellDisplayName = "Rock Bombardment",
	critEffectTargetScript = "sp_shaken_slow_effect",
	Reagents = {
		"Mushrooms",
		},
	Circle = 6,
}

SpellData.AllSpells.Blessing = 
{
	manaCost = 26,
	requireLineOfSight = true,
	effectType = "Projectile",
	MinTravelTime = 0.5,
	PowerWords = "*Jolt*",
	DamageType = "MAGIC",

	Skill = "TheologySkill",
	Circle = 3,
	Icon = "Divine Aura 01",

	Reagents = 
	{
		"Moss","LemonGrass"
	},

	SpellPower = 10,
	SpellRange = SpellData.SpellRanges.Long,
	SpellTravelRate = 60,
	SpellDisplayName = "[CCCC44]Blessing",

	SpellType = "BuffTypeSpell",
	SpellTooltipString = "Bless yor target, improving their strength, agility and intelligence.",

	SpellFireAnim = "spell_fire",

	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",

	SpellFXArgs = "Bone=R_Hand",
	SpellFXDelay = 0.1,
	SpellFXDespawnDelay = 3.0,

	SpellFXName = "HolyEffect",
	SpellHitFX = "SunRayEffect",
	
	SpellPrimeSFX = "event:/magic/fire/magic_fire_cast_fire",
	SpellHitSFX = "event:/magic/fire/magic_fire_fireball_impact",

	CanBeInterrupted = true,
	SpellEnabled = true,
	
	TargetType = "targetMobile",
	TargetRequired = true,
	TargetMobileEffect = "Blessing",
}

SpellData.AllSpells.Heal = 
{
	PowerWords = "Heal",
	SpellPower = 2,
	manaCost = 8,
	effectType = "InstantHitSpell",
	critChance = 5,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	CanClearcast = true,
	critEffectTargetScript = "sp_heal_effect",
	SpellType = "HealTypeSpell",
	SpellPrimeFXName = "ConjurePrimeGreenEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailGreenEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandGreenEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandGreenEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellFXName = "HealEffect",
	SpellPrimeSFX = "event:/magic/water/magic_water_cast_water",
	SpellHitSFX = "event:/magic/misc/magic_water_restoration",
	SpellTooltipString = "Heals the target.",
	SpellFXArgs = "Bone=Ground",
	requireLineOfSight = true,
	Skill = "TheologySkill",
	BeneficialSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "cast_heal",
	SpellDisplayName = "Heal",
	ReagentsRequired = false,
	Reagents = {
		"Ginseng",
	},
	Circle = 2,
}

SpellData.AllSpells.Greaterheal = 
{
	PowerWords = "Greater Heal",
	SpellPower = 10,
	Icon = "restore",
	manaCost = 18,
	effectType = "InstantHitSpell",
	critChance = 5,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	critEffectTargetScript = "sp_heal_effect",
	SpellType = "HealTypeSpell",
	SpellPrimeFXName = "ConjurePrimeGreenEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailGreenEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandGreenEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandGreenEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellFXName = "GreaterHealEffect",
	SpellPrimeSFX = "event:/magic/water/magic_water_cast_water",
	SpellHitSFX = "event:/magic/misc/magic_water_restoration",
	SpellTooltipString = "Greatly heals the target instantly.",
	SpellFXArgs = "Bone=Ground",
	requireLineOfSight = true,
	Skill = "TheologySkill",
	BeneficialSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "cast_heal",
	SpellDisplayName = "Greater Heal",
	ReagentsRequired = true,
	Reagents = {
		"Ginseng",
	},
	Circle = 5,
}

SpellData.AllSpells.Resurrect = 
{
	PowerWords = "Resurrection",
	manaCost = 50,
	effectType = "InstantHitSpell",
	critChance = 1,
	SpellRange = 5,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	TargetMustBeDead = true,
	SpellType = "BeneficialSpellType",
	SpellPrimeFXName = "ConjurePrimeGreenEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailGreenEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandGreenEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandGreenEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellPrimeSFX = "event:/magic/void/magic_void_cast_void",
	SpellHitSFX = "event:/magic/misc/magic_water_restoration",
	SpellTooltipString = "Resurrects the target instantly.",
	requireLineOfSight = true,
	Skill = "TheologySkill",
	BeneficialSpellType = true,
	SpellEnabled = true,
	SpellFireAnim = "cast_heal",
	spellHitEffectTargetScript = "sp_resurrect_effect",
	SpellDisplayName = "Resurrection",
	ReagentsRequired = false,
	Reagents = {
		"Moss",
	},
	Circle = 7,
}

SpellData.AllSpells.Cure = 
{
	PowerWords = "Cure",
	Icon = "regenfocus",
	manaCost = 6,
	effectType = "InstantHitSpell",
	critChance = 0,
	SpellRange = 15,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	SpellType = "BeneficialSpellType",
	SpellFXName = "CureEffect",
	SpellFXArgs = "Bone=Ground",
	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellFXDuration = 3,
	SpellPrimeSFX = "event:/magic/water/magic_water_cast_water",
	SpellHitSFX = "event:/magic/misc/magic_water_restoration",
	SpellTooltipString = "Cures the target of poison.",
	requireLineOfSight = true,
	Skill = "BiomancySkill",
	BeneficialSpellType = true,
	AttackSpellType = false,
	SpellEnabled = true,
	SpellDisplayName = "Cure",
	spellHitEffectTargetScript = "sp_cure_effect",
	ReagentsRequired = false,
	Reagents = {
		"Ginseng",
	},
	Circle = 2,
}

SpellData.AllSpells.Poison = 
{
	PowerWords = "Poison",
	Icon = "Poison Cloud",
	manaCost = 14,
	effectType = "InstantHitSpell",
	SpellPower = 5,
	critChance = 0,
	SpellRange = 12,
	TargetType = "targetMobile",
	TargetRequired = true,
	CanBeInterrupted = true,
	CanClearcast = true,
	SpellType = "MagicAttackTypeSpell",
	SpellPrimeFXName = "ConjurePrimeYellowEffect",
    SpellPrimeFXArgs = "Bone=Spine2",
    SpellPrimeFX2Name = "MagicPrimeTrailYellowEffect",
    SpellPrimeFX2Args = "Bone=L_Hand",
    SpellPrimedFXName = "MagicPrimeHandYellowEffect",
    SpellPrimedFXArgs = "Bone=R_Hand",
    SpellPrimedFX2Name = "MagicPrimeHandYellowEffect",
    SpellPrimedFX2Args = "Bone=L_Hand",
	SpellPrimeSFX = "event:/magic/water/magic_water_cast_water",
	SpellHitSFX = "event:/animals/worm/worm_pain",
	SpellTooltipString = "Poisons the target dealing damage over 21 seconds and reducing healing.",
	requireLineOfSight = true,
	Skill = "BiomancySkill",
	AttackSpellType = true,
	SpellEnabled = true,
	SpellDisplayName = "Poison",
	TargetMobileEffect = "SpellPoison",
	ReagentsRequired = false,
	Reagents = {
		"Mushrooms",
	},
	Circle = 1,
}


require 'spells.pyromancy.static_data.spells'
require 'spells.theology.static_data.spells'