---------------------------------------------------------
-- OVERRIDE FUNCTIONS FOR LOGIN_PLAYER.LUA
---------------------------------------------------------
-- changes we use the core login player functions that
-- are useable by us to change the way they work
---------------------------------------------------------
function GoState(stateName)
	DebugMessage("GoState: "..stateName)
	this:SetObjVar("CharCreateState",stateName)
	if(stateName == "Welcome") then	
		OpenWelcomeWindow()
	elseif(stateName == "PickStartingRace") then	
		OpenStartingRaceWindow()
	elseif(stateName == "PickStartingTemplate") then
		OpenStartingTemplateWindow()
	end
end

function EnterWorld(curCity)
	local regionAddress = GetRegionAddressForName("UpperPlains")
	if( not(IsClusterRegionOnline(regionAddress)) ) then 
		ClientDialog.Show{
				    TargetUser = this,
				    DialogId = "Error",
				    TitleStr = "Error",
				    Button1Str = "Ok",
				    DescStr = "There was an error during character creation. Try again later. If this issue persists, contact an admin.",
				    ResponseFunc = function (user, buttonId)
				    	this:KickUser("Login region not found")
					end
				}
	end		

	this:DelObjVar("CharCreateState")
	this:DelObjVar("CharCreateUniverse")
	-- this tells the player script we were created using the login scene
	this:SetObjVar("CharCreateNew",true)

	local clusterController = GetClusterController()
	if ( clusterController ) then
		clusterController:SendMessage("UserLogin",this,"Connect")
	end

	TeleportUser(this,this,Loc(291,0,892),regionAddress,nil,true,true,true)
end

function GetRegionAddress(cityInfo)
	local universeName = this:GetObjVar("CharCreateUniverse")
	local destRegionAddress = ServerSettings.CharacterCreation.StartingMap
	if(universeName) then
		destRegionAddress = universeName .. "." .. destRegionAddress
	end

	if(cityInfo.Subregion) then
		destRegionAddress = destRegionAddress .. "." ..cityInfo.Subregion
	end

	return destRegionAddress
end
---------------------------------------------------------
-- OVERRIDE EVENT HANDLERS FOR LOGIN_PLAYER.LUA
---------------------------------------------------------
-- we decide to unregister the original event handlers so
-- that they do not interfer with ours as they run in
-- parallel with ours.
---------------------------------------------------------
UnregisterEventHandler(GetCurrentModule(),EventType.ModuleAttached,"")
-- Reregister the module attach event
RegisterEventHandler(EventType.ModuleAttached,GetCurrentModule(), function()
	LoginInit()
	ChangeToTemplate(StartingCharacterBuilds[SelectedCharacterBuild].Template,{KeepAppearance = false, LoadAbilities = false, SetStats = true, SetName = false, BuildHotbar = false, LoadLoot=true, LoadEquipment=true})
	GoState("Welcome")
end)

-- remove the original as we want to override
UnregisterEventHandler(GetCurrentModule(),EventType.LoadedFromBackup,"")
RegisterEventHandler(EventType.LoadedFromBackup,"", function()
	LoginInit()
			
	if not(IsCreate()) then
		-- region is not running give them a choice of new starting region
		ClientDialog.Show{ TargetUser = this, DialogId = "Error", TitleStr = "Error", Button1Str = "Ok", DescStr = "The region your character logged out in is not online. Please select a starting town.", ResponseFunc = function (user, buttonId) end }
		GoState("Universe")
	else
		local createState = this:GetObjVar("CharCreateState")
		--DebugMessage("CREATE STATE: "..tostring(createState))
		if not(createState) then
			DebugMessage("ERROR: Player entered world without createState.")
			ChangeToTemplate(StartingCharacterBuilds[SelectedCharacterBuild].Template,{KeepAppearance = false, LoadAbilities = false, SetStats = true, SetName = false, BuildHotbar = false, LoadLoot=true, LoadEquipment=true})
			GoState("Welcome")
		else
			ChangeToTemplate(StartingCharacterBuilds[SelectedCharacterBuild].Template,{KeepAppearance = false, LoadAbilities = false, SetStats = true, SetName = false, BuildHotbar = false, LoadLoot=true, LoadEquipment=true})
			GoState("Welcome")
		end
	end
end)

-- just in case we need this and it gets changed
UnregisterEventHandler(GetCurrentModule(),EventType.UserLogout,"")
RegisterEventHandler(EventType.UserLogout,"", function()this:CompleteLogout() end)

