require 'default:player'
require 'player_world_building'
require 'player_dungeon_editor'
-- OVERRIDE
--* reworking how the region status is displayed
function PerformPlayerShortTick()
	UpdatePlayerProtection(this)

	--?? Rework how region names are displayed for custom maps
	local newRegionalName = GetRegionalName(this:GetLoc())
	if(newRegionalName ~= nil and newRegionalName ~= currentRegionalName) then
		-- currentRegionalName = newRegionalName
		-- if(currentRegionalName == "TheLanding2") then currentRegionalName = "The Expanse" end
		-- this:SystemMessage("You have entered "..currentRegionalName,"event")
		-- this:SendMessage("EnteredRegionalName", currentRegionalName)
		UpdateRegionStatus(this,currentRegionalName)
	end
end


-- OVERRIDE
--* removal of missions,vitatlity checks
function PerformPlayerTick(notFirst, playMinutes)
	ProfessionsHelpers.ApplyTraits( this )
	
	-- prevent logins and reloads taking minutes away from initiate status
	if ( notFirst ) then
		-- check initiate
		CheckInitiate(this)
	end

	CheckBidRefund()
	CheckGmMessage(this)
	TryPenitentExpire( this )
	UpdateMurderCountExpire(this, playMinutes)

	-- Checks every minute if challenge quests can be re-started
	ChallengeSystemHelper.DoUpkeep( this )

end

function OnLoadAddThings()
	if not(this:HasObjVar("MagicKnowledge")) then
		local magicKnowledge = {}
		magicKnowledge.ElectromancySkill = {}
		magicKnowledge.PyromancySkill = {}
		magicKnowledge.TheologySkill = {}
		magicKnowledge.BiomancySkill = {}
		magicKnowledge.ConjurationSkill = {}
		this:SetObjVar("MagicKnowledge",magicKnowledge)
	end
	if not(this:HasObjVar("MountCollection")) then
		local mountCollection = {}
		mountCollection.Horses = {}
		mountCollection.Llamas = {}
		mountCollection.Bears = {}
		mountCollection.Misc = {}
		this:SetObjVar("MountCollection",mountCollection)
	end

	if not(this:HasObjVar("CanUseMounts")) then
		this:SetObjVar("CanUseMounts",false)
	end	
	-- this fixes the players backpack weight problem to many call function delays going on around this server
	CallFunctionDelayed(TimeSpan.FromSeconds(1),function() RecalculateMaxWeight() end)
end

RegisterEventHandler(EventType.LoadedFromBackup,"",OnLoadAddThings)

-- This gets called on both creation and loading from backup
function OnLoad(isPossessed)
	if not(isPossessed) then
		if(not(this:HasObjVar("playerInitialized"))) then
			InitializePlayer()
			this:SetObjVar("playerInitialized",true)
		end

		--DAB/DFB hack: If the position is not valid send them to the outcast spawn position.
		if (not this:GetLoc():IsValid()) then
			local spawnPosition = FindObjectWithTag("OutcastSpawnPosition")
			if (spawnPosition == nil) then
				this:SetWorldPosition(GetPlayerSpawnPosition(this))
			else
				this:SetWorldPosition(spawnPosition:GetLoc())
			end
		end

		-- Check if player will be logged-in to a Stranger's House and kick them off plot, if so.
		Plot.KickIfStranger(nil, this)

		this:SetObjVar("LoginTime",DateTime.UtcNow)
		bankBox = this:GetEquippedObject("Bank")
		if( bankBox == nil ) then
			CreateEquippedObj("bank_box", this)
		end

		tempPack = this:GetEquippedObject("TempPack")
		if( tempPack == nil ) then
			CreateEquippedObj("crate_empty", this, "temppack_created")
			-- create the key ring inside the temp pack
			RegisterSingleEventHandler(EventType.CreatedObject,"temppack_created",
				function (success,objRef)
					objRef:SetSharedObjectProperty("Weight",-1)
					objRef:SetName("Internal Temp Pack")
					CheckKeyRing(objRef)
				end)
		else
			CheckKeyRing()
		end	
	
		ShowSkillTracker()
		if not this:HasObjVar("LifetimePlayerStats") then
			
			local newLifetimeStats = 
			{
			Players = 0,
			TotalMonsterKills = 0,
			}

			this:SetObjVar("LifetimePlayerStats",newLifetimeStats)
		end

		this:SendClientMessage("PlayerRunSpeeds",
			{
				ServerSettings.Stats.WalkSpeedModifier,
				ServerSettings.Stats.RunSpeedModifier
			})
		this:SetMaxMoveSpeedModifier(ServerSettings.Stats.RunSpeedModifier)

		--UpdateFactions()

		this:SetStatValue("PrestigeXPMax",ServerSettings.Prestige.PrestigePointXP)

		if ( this:GetSharedObjectProperty("IsSneaking") ) then
			StartMobileEffect(this, "Hide", nil, {Force = true})
		end

	else
		this:SendClientMessage("PlayerRunSpeeds",
			{
				1.0,
				this:GetObjVar("AI-ChargeSpeed") or 2.0
			})
	end

	this:SetObjectTag("AttachedUser")

	-- send skill values to player
	SendSkillList()

	UpdateClientSkill(this,this)

	--refresh the quest UI
	--this:SendMessage("RefreshQuestUI")	

	SendTimeUpdate(this)	

	InitializeHotbar()

	if(UpdateWeatherViews) then
		UpdateWeatherViews()
	end

    this:SendMessage("LoggedIn")
	
	-- DAB TODO: Sometimes EnterView will fire on another object (teleporter) before this fires
	-- some scripts might not want to act on someone who has just entered the world
	this:ScheduleTimerDelay(TimeSpan.FromSeconds(5),"EnteringWorld")

	this:DelObjVar("BuffIcons")

	--ApplyPersistentMobileEffects(this)

	-- Delayed OnLoadStuff
	CallFunctionDelayed(TimeSpan.FromSeconds(0.5),
		function()		
			if(initializer and initializer.HotbarActions) then
				BuildHotbar(initializer.HotbarActions)
			end						

			-- These functions are found in globals/dynamic_window/hud
			UpdateHotbar(this)
			UpdateSpellBar(this)
			UpdateItemBar(this)
			UpdateBloodlust(this)
			ShowStatusElement(this,{IsSelf=true,ScreenX=10,ScreenY=10})

			-- this technically does not need to be called every time you come from the backup
			UpdateFixedAbilitySlots()

			if not( isPossessed ) then
				InitializeClientConflicts(this)
				UpdateName()

				if ( IsMounted(this) and this:IsInRegion("NoMount") ) then
					DismountMobile(this)
				end			
			end
		end)

	-- GW TODO we need better handling of client messages in general, for ones that expect client objects to be constructed and started by time they are fired.
	CallFunctionDelayed(TimeSpan.FromSeconds(10.0),
		function()		
			if this:HasObjVar("AchievementWaiting") then
				this:SendClientMessage("SetAchievementNotification",true)
			end
		end)

	local equippedObjects = this:GetAllEquippedObjects()
	for i=1,#equippedObjects do
		if not(equippedObjects[i]:GetCreationTemplateId()) then
			equippedObjects[i]:Destroy()
			DebugMessage("ERROR: Destroying invalid equipped item. "..equippedObjects[i].Id)
		end
	end

	this:ScheduleTimerDelay(TimeSpan.FromSeconds(5 + math.random()),"UpdateChatChannels")

	Militia.UpdatePlayerVars(this)
	Militia.UpdateTitle(this)

	UpdatePlayerProtection(this,true)

	--update quest map markers
	local activeQuests = Quests.GetActive(this)
	for questName, quest in pairs(activeQuests) do
		Quests.UpdateQuestMarkers(this, questName, true)
	end

	this:DelObjVar("HomeUniverse")
end

RegisterEventHandler(EventType.Timer, "cacheMobileMods", function()
	this:SetObjVar("CachedMobileMods",MobileMod)
	this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(350), "cacheMobileMods")
end)

this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(350), "cacheMobileMods")

function DoUse(usedObject,usedType)
	if (usedObject == nil or not usedObject:IsValid()) then return end
	
	if (not(IsInCombat(this)) and usedObject:HasModule("merchant_sale_item") == true and usedType == "Attack") then
		usedType = usedObject:GetSharedObjectProperty("DefaultInteraction")
	end

	--If the object is in a locked container, players cannot use it
	if (usedType ~= "God Info" and usedObject:IsInContainer()) then
		local container = usedObject:ContainedBy()
		local key = GetKey(this,container)
		if (container:GetObjVar("locked")) then
			-- Secure Containers allow a user to open and view contents without unlocking and rendering the container vulnerable
			if ( not container:HasObjVar("SecureContainer") or not Plot.HasObjectControl(this, container, container:HasObjVar("FriendContainer")) ) then
				this:SystemMessage("That is in a locked container.","info")
				return
			end
		end
	end

	--Load the use case ranges from use_cases.lua	
	local usedCaseRange = OBJECT_INTERACTION_RANGE
	local topmost = usedObject

	if not(usedObject:IsPermanent()) then
		if( IsDead(this) and not usedObject:HasObjVar("UseableWhileDead")) then
			return
		end

		topmost = usedObject:TopmostContainer() or usedObject
		if (AllUseCases[usedType] ~= nil) then
			usedCaseRange = AllUseCases[usedType].Range or OBJECT_INTERACTION_RANGE
			if (AllUseCases[usedType].Restriction == "God" and not IsGod(this)) then
				DebugMessage("WARNING: Player "..this:GetName().." attempted to use a use case restricted to gods! usedType is "..tostring(usedType))
				return
			end
			if (AllUseCases[usedType].Restriction == "Immortal" and not IsGod(this)) then
				DebugMessage("WARNING: Player "..this:GetName().." attempted to use a use case restricted to immortals! usedType is "..tostring(usedType))
				return
			end
			if (AllUseCases[usedType].Restriction == "DemiGod" and not IsGod(this)) then
				DebugMessage("WARNING: Player "..this:GetName().." attempted to use a use case restricted to demigods! usedType is "..tostring(usedType))
				return
			end
		end

		if(IsInTradeContainer(usedObject) and (not(usedObject:IsContainer()) or usedType ~= "Open")) then
			return
		end
	else
		if( IsDead(this) ) then
			return
		end
	end

	if( HelperHiredMerchants.IsHiredMerchant(usedObject) and usedType == 'Interact' ) then
		usedCaseRange = AllUseCases["Inspect"].Range
	end

	if (this:GetLoc():Distance2(topmost:GetLoc()) > usedCaseRange and not IsGod(this)) then
		this:SystemMessage("You are too far to do that.","info")
		return 
	end

	--DebugMessage("DoUse: "..tostring(usedObject)..", "..tostring(usedType))
	if(usedObject:IsValid()) then
		if(usedObject:IsPermanent()) then			
			--DebugMessage("DoUsePermanent",this:GetName(),tostring(usedObject),tostring(usedObject:GetSharedObjectProperty("ResourceSourceId")),tostring(usedType))

			-- right now the only thing you can do with permanents is harvest
			TryHarvestItem(usedObject)
		else
			if(usedType == nil or usedType == "") then
				usedType = usedObject:GetSharedObjectProperty("DefaultInteraction") or "Use"
			end

			--DebugMessage("DoUse",this:GetName(),tostring(usedObject),usedObject:GetName(),tostring(usedType))
			if(usedType == "Character") then
				this:SendMessage("OpenCharacterWindow")
			elseif(usedType == "Stand") then
				if (IsSitting(this)) then
					this:SendMessage("StopSitting")
					this:SetWorldPosition(this:GetObjVar("PositionBeforeUsing"))
				end
				RemoveUseCase(this, "Stand")
			elseif(usedType == "Wake Up") then
				this:SendMessage("WakeUp")
				this:SetWorldPosition(this:GetObjVar("PositionBeforeUsing"))
				RemoveUseCase(this, "Wake Up")
			elseif(usedType == "Inspect" or usedType == "Disrobe") then
				OpenInspectWindow(usedObject)
			elseif (usedType == "Kick from Group") then
				local groupId = GetGroupId(usedObject)
				if ( groupId ~= nil and GetGroupVar(groupId, "Leader") == this ) then
					GroupRemoveMember(groupId, usedObject)
				end
			elseif (usedType == "Invite to Group") then
				GroupInvite(this, usedObject)
			elseif (usedType == "Group") then
				if ( this:HasModule("group_ui") ) then
					this:SendMessage("GroupUpdate")
				else
					this:AddModule("group_ui")
				end
			elseif (usedType == "Invite to Guild") then
				Guild.Invite(this, usedObject)				
			elseif (usedType == "Send Friend Request") then
				SendFriendRequest(this, usedObject)
			elseif (usedType == "Trade") then
				if(IsInActiveTrade(this)) then
					this:SystemMessage("You already have an active trade in progress.","info")
				elseif(IsInActiveTrade(usedObject)) then
					this:SystemMessage("The target already has an active trade in progress.","info")
				elseif(IsInCombat(this)) then
					this:SystemMessage("Cannot trade while in combat.","info")
				elseif(IsInCombat(usedObject)) then
					this:SystemMessage("The target is in combat.","info")
				else
					this:AddModule("trading_controller",{TradeTarget=usedObject})
				end
			-- Runebooks
			elseif (usedType == "Rename Runebook") then
				StartMobileEffect(usedObject, "RunebookRename", this)
			-- Gardening Use Types
			elseif (usedType == "Water") then
				StartMobileEffect(usedObject, "WaterPlant", this)
			elseif (usedType == "Gather") then
				StartMobileEffect(usedObject, "GatherPlant", this)
			elseif (usedType == "Empty") then
				StartMobileEffect(usedObject, "EmptyPlant", this)
			elseif (usedType == "Force Grow") then
				if (IsGod(this)) then
					Gardening.DoGrowth(usedObject, true)
				end
			-- A player wants to listen to a band
			elseif (usedType == "Listen to Music") then
				StartMobileEffect(this, "ListenToBand", usedObject)
			elseif (usedType == "Stop Listening") then
				this:DelObjVar("ListeningToBandID")
			elseif(usedType == "Give Tip") then
				StartMobileEffect(this, "TipBard", usedObject)
			elseif (usedType == "Duel") then
				Duel.Challenge(this, usedObject)
			elseif (usedType == "Loot All" or usedType == "Loot Gold") then
				local targetContainer = usedObject
				if (targetContainer:IsMobile()) then	
					local backpackObj = targetContainer:GetEquippedObject("Backpack")    
					if(backpackObj) then
						targetContainer = backpackObj
					end
				elseif (targetContainer:IsContainer()) then
					if(targetContainer:HasObjVar("locked")) then
						if ( not targetContainer:HasObjVar("SecureContainer") or not Plot.HasObjectControl(this, targetContainer, targetContainer:HasObjVar("FriendContainer")) ) then
				    	   this:SystemMessage("That appears to be locked.","info")
				    	   return
				        end
				    end
				end
				if( usedType == "Loot All" ) then
					StartLootAll(targetContainer)
				else
					StartLootAll(targetContainer, true)
				end
			elseif (usedType == "God Info") then
				if (IsGod(this)) then
					DoInfo(usedObject)
				else
					DebugMessage("WARNING: Player "..this:GetName().." attempted to open a god info window for "..usedObject:GetName()..", player is not a God character.")
				end
			elseif (usedType == "Manage Arena") then
				if (IsGod(this)) then
					this:SendMessage("StartMobileEffect", "GodManageRankedArena", usedObject)
				else
					DebugMessage("WARNING: Player "..this:GetName().." attempted to open a manage arena window for "..usedObject:GetName()..", player is not a God character.")
				end
			elseif (usedType == "Manage Dynamic Spawn") then
				if (IsGod(this)) then
					this:SendMessage("StartMobileEffect", "GodManageDynamicSpawn", usedObject)
				end
			elseif (usedType == "Edit Mob") then
				if(IsGod(this)) then
					DoMobEdit(usedObject)
				end
			elseif(usedType == "Pick Up" or usedType == "God Pick Up") then
				HandleRequestPickUp(usedObject)
			elseif(usedType == "Quick Loot") then
				ProgressBar.Show{TargetUser=this,Label="Looting",Duration=AUTOLOOT_DELAY}
				this:ScheduleTimerDelay(TimeSpan.FromSeconds(AUTOLOOT_DELAY),"autolootitem",usedObject,usedObject:ContainedBy())
			elseif(usedType == "Harvest" or usedType == "Chop" or usedType == "Mine" or usedType == "Skin" or usedType == "Forage") then
				TryHarvestItem(usedObject)
			-- DAB TODO: Handle gods and subordinates
			elseif(usedType == "Equip") then
				if(usedObject:TopmostContainer() ~= this) then
					this:SystemMessage("[$2404]","info")
				else
					DoEquip(usedObject,this)
				end
			elseif (usedType == "Tame") then
				if not( HasMobileEffect(this, "Tame") ) then
					StartMobileEffect(this, "Tame", usedObject)
				end
			elseif (usedType == "Attack") then
				--DebugMessage("Getting here.")
				this:SendMessage("AttackTarget",usedObject)
			elseif(usedType == "Unequip") then
				if(usedObject:IsEquippedOn(this)) then
					user:SystemMessage("You can only unequip items from yourself.","info")
				else
					DoUnequip(usedObject,this)
				end
			-- always try to harvest objects with a resource source id
			elseif(usedObject:HasObjVar("ResourceSourceId") and usedType == "Use") then
				if not(TryHarvestItem(usedObject)) then
					usedObject:SendMessage("UseObject",this,usedType)
				end
			elseif( ValidResourceUseCase(usedObject, usedType) ) then
				TryUseResource(this, usedObject, usedType)
			
			elseif(usedType=="Equipment Editor") then
	    		if not (usedObject:HasModule("equipment_editor.equipment_editor")) then
		    		usedObject:AddModule("equipment_editor.equipment_editor")
	    		end
	    		usedObject:SendMessage("OpenForUser",this)
	      		this:CloseDynamicWindow("ColorEditorDynamicResponse")
	    		this:CloseDynamicWindow("EssenceEditorDynamicResponse")				
			elseif(usedType=="Enchantment Information") then
				ShowItemPropertyWindow(this,usedObject)
			elseif(usedType=="Clip Seed Information") then
				HandleClipTarget(usedObject,this)
			else
				usedObject:SendMessage("UseObject",this,usedType)
			end
		end		
	end
end



