---
-- @param user - (GameObject) the admin using this dungeon editor
function ShowDungeonEditor(user)
	-- create local objects and variables
	-- used for color formatting in the dungeon editor dynamic window
	local switchcolors = true
	local tileTextColor = "[959555]"
	local userDungeonData = user:GetObjVar("DungeonEditor")

	-- first time dungeon? lets create everything to get started
	if(userDungeonData == nil) then 
		CreateNewDungeonWindow(user)
		return		
	end
	local dungeonData = GetDungeonDataByName(user,user:GetObjVar("DungeonEditor").SelectedDungeon)
	local dungeonWidth = dungeonData.Width or 10
	local dungeonHeight = dungeonData.Height or 10
	local selectedTile = userDungeonData.SelectedTile	
	--TODO: Clean up offsets to reduce math calls    
    local xOffset = 0
    local yOffset = 30

    -- dynamic window objects
	local dynamicWindow = DynamicWindow("DungeonEditorWindow","",1000,600,-500,-300,"TransparentDraggable","Center")
	local scrollWindow = ScrollWindow(700,110,280,500,50)
    local scrollElement = nil 

    -- create the main background window
    dynamicWindow:AddImage(1,1,"HueSwatch",1000,630,"Simple","252535")
    dynamicWindow:AddImage(3,3,"HueSwatch",994,624,"Simple","151525")
    -- create the header background and text
    dynamicWindow:AddImage(1,1,"HueSwatch",1000,34,"Simple","353545")
    dynamicWindow:AddImage(-2,-2,"CraftingItemsFrame",1006,40,"Sliced","557575")
    dynamicWindow:AddLabel(510,10,"[959555]Dungeon Editor - (".. userDungeonData.SelectedDungeon..")["..dungeonWidth..","..dungeonHeight.."]",980,20,20,"center")	
    dynamicWindow:AddImage(690,-60,"VerticalWindowDivider",3,790,"Simple","959555")

    -- current selected tiled
    DrawDungeonIcon(701,44,dynamicWindow,GetTileDataByName(selectedTile).Icon,5)

    -- draw the dungeon layout
    local x = 1
    local y = 1
    local tileData = {}

    dynamicWindow:AddImage(20,19+yOffset,"HueSwatch",dungeonWidth*20+4,dungeonHeight*20+3,"Simple","090919")

    for index=1,tonumber(dungeonWidth)*tonumber(dungeonHeight) do
    	if(x > tonumber(dungeonWidth)) then 
    		x = 1 
    		y = y + 1
    	end
    	--print(x,y,dungeonData.DungeonData[index])
    	if(dungeonData.DungeonData[index] ~= "A Empty Tile") then
    		tileData = GetTileDataByName(dungeonData.DungeonData[index])
			DrawDungeonIcon(x*20+xOffset,y*20+yOffset,dynamicWindow,tileData.Icon,2)
		end
		dynamicWindow:AddButton(x*20+xOffset,y*20+yOffset,"CellClicked|"..index,"",20,20,"Cell (" .. x .. ",".. y .. ")","",false,"Invisible")    	
    	x = x + 1
    end

    -- tiles selection
	for tileIndex,tileValue in pairs(TileSets.StoneAndBone.Tiles) do
		scrollElement = ScrollElement()
	    if(switchcolors== false) then
	        color="252535"
	    else
	        color="151525"
	    end    		
		scrollElement:AddImage(0,0,"HueSwatch",250,40,"Simple",color)
		if(tileValue.Name == selectedTile.Name) then tileTextColor = "[FF7700]" end

		scrollElement:AddLabel(54,2,tileTextColor .. tileValue.Name,230,16,16,"left")
		scrollElement:AddLabel(54,18,"[757533]" .. tileValue.Template,230,14,14,"left")
		DrawDungeonIcon(3,5,scrollElement,tileValue.Icon,3)
		--scrollElement:AddButton(0,0,"PlaceItem|"..tileValue.Template.."|"..tileValue.Name,"",235,20,"Click to place " .. tileValue.Name,"",false,"Invisible")
		scrollElement:AddButton(5,5,"SelectTile|"..tileValue.Name,"",240,40,"[FF7700]Tile Selection\n[959555]Click to select this tile","",false,"Invisible")
		scrollWindow:Add(scrollElement)
		switchcolors = not switchcolors 
		tileTextColor = "[959555]"
	end

	-- button toolbar
	dynamicWindow:AddImage(20,585,"HueSwatch",150,25,"Simple","252535")
	dynamicWindow:AddImage(22,587,"HueSwatch",146,21,"Simple","202030")    		
	dynamicWindow:AddLabel(95,588,"[959555]Create",150,24,24,"center")
	dynamicWindow:AddButton(20,585,"CreateDungeon|","",150,25,"[FF7700]Create A New Dungeon[959555]\nClicking this will open up a dialog window to enter the attributes for the new dungeon you want to design for your adventures.","",true,"Invisible")	

	dynamicWindow:AddImage(180,585,"HueSwatch",150,25,"Simple","252535")
	dynamicWindow:AddImage(182,587,"HueSwatch",146,21,"Simple","202030")    		
	dynamicWindow:AddLabel(255,588,"[959555]Open",150,24,24,"center")
	dynamicWindow:AddButton(180,585,"OpenDungeon|","",150,25,"[FF7700]Open A Previous Dungeon[959555]\nClicking this will open up a dialog window to select a previous dungeon you created.","",true,"Invisible")	

	dynamicWindow:AddImage(340,585,"HueSwatch",150,25,"Simple","252535")
	dynamicWindow:AddImage(342,587,"HueSwatch",146,21,"Simple","202030")    		
	dynamicWindow:AddLabel(415,588,"[959555]Script It",150,24,24,"center")
	dynamicWindow:AddButton(340,585,"ScriptIt|","",150,25,"[FF7700]Script The Dungeon Controller[959555]\nClicking this will generate the controller that will populate this dungeon at the location you place the controller. Make sure you /reloadtemplates prior to creating your new dungeon controller in game..","",false,"Invisible")	

	dynamicWindow:AddImage(500,585,"HueSwatch",150,25,"Simple","252535")
	dynamicWindow:AddImage(502,587,"HueSwatch",146,21,"Simple","202030")    		
	dynamicWindow:AddLabel(575,588,"[959555]Clip It",150,24,24,"center")
	dynamicWindow:AddButton(500,585,"ClipIt|","",150,25,"[FF7700]Clip Dungeon Seed Information[959555]\nClicking this will format the dungeon information into a permanent dungeon in your world. it will copy the data to your clipboard so you can paste it in your maps seed object file.","",false,"Invisible")	

	--attach the scrollWindow object to the dynamic window
	dynamicWindow:AddScrollWindow(scrollWindow)
	-- draw the close button on the dynamic window
	DrawCloseButton(971,9,dynamicWindow)

	-- open up the finalized dynamic window for the user
	user:OpenDynamicWindow(dynamicWindow,user)
end

---
-- @param x - (Number) the x location to draw the icon at
-- @param y - (Number) the y location to draw the icon at
-- @param element - the dynamic window or scroll element  you want to apply this icon too
-- @param iconData - (Table) an array of 100 meant to be tiled in a 10x10 format
-- @param scale - (Number) the scale to draw the icon, make sure to use whole numbers 2,3,4,5
function DrawDungeonIcon(x,y,element,iconData,scale)
	scale = scale or 1
	local count = 1
	local color = "777777"
	for i=1,#iconData do
		-- walls
		if(iconData[i] == 1) then color = "FF3333" end
		-- floor
		if(iconData[i] == 2) then color = "303030" end
		if(count > 10) then
			count = 1
			y = y + scale
		end
		if(iconData[i] ~= 0) then
			element:AddImage(x+count*scale,y,"HueSwatch",scale,scale,"Simple",color)
		end
		count = count + 1
	end
end

--- this will draw the close button on a dynamic window
-- @param x - this is the x location to draw the close button at
-- @param y - this is the y location to draw the close button at
-- @param dynamicWindow - this is the dynamic window to attach the close button too
function DrawCloseButton(x,y,dynamicWindow)
	--border background color
	dynamicWindow:AddImage(x,y,"Blank",18,18,"Sliced","555555")
	--background color
	dynamicWindow:AddImage(x+2,y+2,"Blank",14,14,"Sliced","252525")
	--invisible button
	dynamicWindow:AddButton(x,y,"CloseButton|ParentForm","",18,18,"[CCCC33]Close Button\n[FFFFFF]clicking this will close the window.","",true,"Invisible")
	--draw the "X" string
	dynamicWindow:AddLabel(x+5,y+3,"[959555]X",18,18,18,"left")
end

---
-- @param user - (GameObject) the admin using this dungeon editor
-- @param buttonId - (String) the string handler for the button that was pressed
-- @param textField - (Table) holding the textfiled data in your dynamic windows
function HandleDungeonEditorWindowResponse(user,buttonId,textField)
	local args = StringSplit(buttonId, "|")
	local button = args[1]
	
	-- dungeon editor
	if(button == "SelectTile") then
		local userDungeonData = user:GetObjVar("DungeonEditor")
		userDungeonData.SelectedTile = args[2]
		user:SetObjVar("DungeonEditor",userDungeonData)
		ShowDungeonEditor(user) 
	end
	if(button == "CreateDungeon") then
		CreateNewDungeonWindow(user)
	end
	if(button == "CellClicked") then
		local userDungeonData = user:GetObjVar("DungeonEditor")
		local selectedDungeon = userDungeonData.SelectedDungeon
		local selectedTile = userDungeonData.SelectedTile
		local cellIndex = tonumber(args[2])
		local dungeonIndex = GetDungeonIndex(user,selectedDungeon)
		local dungeonData = GetDungeonDataByName(user,selectedDungeon)
		print("dungeonindex:",dungeonIndex)
		print("dungeonData:",dungeonData.Name)
		print("selectTile:",selectedTile)
		print("selectedDungeon:",selectedDungeon)
		print("cellIndex:",cellIndex)
		userDungeonData.Dungeons[dungeonIndex].DungeonData[cellIndex] = selectedTile
		user:SetObjVar("DungeonEditor",userDungeonData)
		ShowDungeonEditor(user)
	end
	if(button == "OpenDungeon") then
		OpenDungeonWindow(user)
	end
	if(button == "ScriptIt") then
		SaveDungeonToController(user)
	end

	-- create new dungeon dialog
	if(button == "CreateNewDungeonDialog") then
		local name = textField.Name
		local template = textField.Template 
		local height = textField.Height
		local width = textField.Width
		IfNoUserData(user)
		CreateEmptyDungeon(user,{Name=name,Template=template,Height=height,Width=width})
		ShowDungeonEditor(user)
	end

	-- open dungeon dialog
	if(button == "SelectDungeonToEdit") then
		local selectedDungeon = args[2]
		local userDungeonData = user:GetObjVar("DungeonEditor")
		userDungeonData.SelectedDungeon = selectedDungeon
		user:SetObjVar("DungeonEditor",userDungeonData)
		ShowDungeonEditor(user)
	end
end

---
-- @param user - (GameObject) the admin using this dungeon editor
-- @param dungeonData - (Table) this information is generated in the handledungeoneditorwindowresponse function when the
--                              user clicks the create button for adding a new dungeon
function CreateEmptyDungeon(user,dungeonData)
	local dungeonTable = 
	{
		Name = dungeonData.Name,
		Template = dungeonData.Template,
		Height = dungeonData.Height,
		Width = dungeonData.Width,
		DungeonData = {}
	}

	local dungeonTileData = {}
	for index=1,dungeonTable.Height * dungeonTable.Width do
		dungeonTileData[index] = "A Empty Tile"
	end
	dungeonTable.DungeonData = dungeonTileData
	
	local dungeonEditorTable = user:GetObjVar("DungeonEditor")
	dungeonEditorTable.SelectedDungeon = dungeonTable.Name
	dungeonEditorTable.SelectedTile = "A Empty Tile"
	
	
	table.insert(dungeonEditorTable.Dungeons,dungeonTable)
	
	user:SetObjVar("DungeonEditor",dungeonEditorTable)
end


---
-- @param user - (GameObject) the admin using this dungeon editor
function CreateNewDungeonWindow(user)
    -- dynamic window objects
	local dynamicWindow = DynamicWindow("DungeonEditorWindow","",500,280,-250,-140,"TransparentDraggable","Center")

    -- create the main background window
    dynamicWindow:AddImage(1,1,"HueSwatch",500,280,"Simple","252535")
    dynamicWindow:AddImage(3,3,"HueSwatch",494,274,"Simple","151525")
    -- create the header background and text
    dynamicWindow:AddImage(1,1,"HueSwatch",500,34,"Simple","353545")
    dynamicWindow:AddImage(-2,-2,"CraftingItemsFrame",506,40,"Sliced","557575")
    dynamicWindow:AddLabel(260,10,"[959555]New Dungeon Dialog",480,20,20,"center")	

    dynamicWindow:AddImage(-2,30,"CraftingItemsFrame",506,110,"Sliced","557575")
    dynamicWindow:AddLabel(15,34,"[959555]Below is a list of variables required to build your dungeon. You can change the width to a max of 32 and the height to a max of 25 as this is as big as the UI supports. You will also need to supply a unique dungeon name and a unique template name if you plan on saving this dungeon to a dungeon_controller object.",470,100,20)

    dynamicWindow:AddLabel(15,150,"[959555]Height",100,20,20,"left")
    dynamicWindow:AddTextField(100,150,380,20,"Height","10")

    dynamicWindow:AddLabel(15,170,"[959555]Width",100,20,20,"left")
    dynamicWindow:AddTextField(100,170,380,20,"Width","10")

    dynamicWindow:AddLabel(15,190,"[959555]Name",100,20,20,"left")
    dynamicWindow:AddTextField(100,190,380,20,"Name","")

    dynamicWindow:AddLabel(15,210,"[959555]Template",100,20,20,"left")
    dynamicWindow:AddTextField(100,210,380,20,"Template","")

	dynamicWindow:AddImage(15,240,"HueSwatch",470,25,"Simple","252535")
	dynamicWindow:AddImage(17,242,"HueSwatch",466,21,"Simple","202030")    		
	dynamicWindow:AddLabel(250,243,"[959555]Create New Dungeon",470,24,24,"center")
    dynamicWindow:AddButton(15,240,"CreateNewDungeonDialog|","",470,25,"[FF7700]Create This Empty Dungeon[959555]\nClicking this will generate a new empty dungeon that you can use in the dungeon editor.","",true,"Invisible")	


	-- draw the close button on the dynamic window
	DrawCloseButton(471,9,dynamicWindow)

	-- open up the finalized dynamic window for the user
	user:OpenDynamicWindow(dynamicWindow,user)	
end

---
-- @param user - (GameObject) the admin using this dungeon editor
function OpenDungeonWindow(user)
    -- dynamic window objects
	local dynamicWindow = DynamicWindow("DungeonEditorWindow","",500,500,-250,-250,"TransparentDraggable","Center")
	local scrollWindow = ScrollWindow(10,90,470,400,20)
    local scrollElement = nil 
    local switchcolors = false
	local dungeonEditorTable = user:GetObjVar("DungeonEditor")

    -- create the main background window
    dynamicWindow:AddImage(1,1,"HueSwatch",500,500,"Simple","252535")
    dynamicWindow:AddImage(3,3,"HueSwatch",494,494,"Simple","151525")
    -- create the header background and text
    dynamicWindow:AddImage(1,1,"HueSwatch",500,34,"Simple","353545")
    dynamicWindow:AddImage(-2,-2,"CraftingItemsFrame",506,40,"Sliced","557575")
    dynamicWindow:AddLabel(260,10,"[959555]Select A Dungeon To Edit",480,20,20,"center")	

    dynamicWindow:AddImage(-2,30,"CraftingItemsFrame",506,50,"Sliced","557575")
    dynamicWindow:AddLabel(15,34,"[959555]Below is a list of dungeons you have created. Use the interface to select a dungen to edit or delete.",470,100,20)

    table.sort(dungeonEditorTable.Dungeons,function (a,b)
		return a.Name < b.Name
	end)

    for k,v in pairs(dungeonEditorTable.Dungeons) do
    	if(switchcolors== false) then
	        color="252535"
	    else
	        color="151525"
	    end 
    	scrollElement = ScrollElement()
    	scrollElement:AddImage(1,1,"HueSwatch",450,20,"Simple",color)
    	scrollElement:AddLabel(5,1,"[959555]" .. v.Name,200,20,20,"left")

    	scrollElement:AddLabel(260,3,"[959555] Size [".. v.Height .. "," .. v.Width .. "]",80,20,16,"left")

    	scrollElement:AddImage(390,1,"HueSwatch",60,20,"Simple","252525")
    	scrollElement:AddImage(392,3,"HueSwatch",56,16,"Simple","151515")
    	scrollElement:AddLabel(420,3,"[959555]Edit",60,16,16,"center")
 	    scrollElement:AddButton(390,2,"SelectDungeonToEdit|"..v.Name,"",60,18,"[FF7700]Select Dungeon To Edit[959555]\nClicking this will select the dungeon you want to work with when you open up the dungeon editor.","",true,"Invisible")	

    	scrollWindow:Add(scrollElement)
    	switchcolors = not switchcolors 
    end

	--attach the scrollWindow object to the dynamic window
	dynamicWindow:AddScrollWindow(scrollWindow)

	-- draw the close button on the dynamic window
	DrawCloseButton(471,9,dynamicWindow)

	-- open up the finalized dynamic window for the user
	user:OpenDynamicWindow(dynamicWindow,user)	
end

-- event handlers
-- when you use any button or response object on your dynamic window for the dunge editor this event function will fire
RegisterEventHandler(EventType.DynamicWindowResponse,"DungeonEditorWindow", HandleDungeonEditorWindowResponse)
-- this handles the event from the player hud script that shows the quick access button thats a part of the admin toolbar
RegisterEventHandler(EventType.ClientUserCommand, "ToggleShowDungeonEditor", function ()
	ShowDungeonEditor(this)
end)