-- COMPLETE OVERRIDE OF WHOLE SYSTEM

require 'incl_gametime'
require 'spawnerprofiles.main'

isDungeon = IsDungeonMap()


-- the delay between spawn attempts, can be overridden by adding "spawnDelay" objvar
DEFAULT_DELAY_SECS = 180
-- the delay between spawn amounts, can be overridden by adding "spawnCount" objvar
DEFAULT_SPAWN_COUNT = 1
DEFAULT_SPAWN_CHANCE = 1
DEFAULT_PULSE_FREQ_SECS = 5
mSpawnPower = nil

mIsPendingCreate = false

function GetSpawnPulse()    
    local delaySecs = this:GetObjVar("spawnPulseSecs") or DEFAULT_PULSE_FREQ_SECS
    return TimeSpan.FromSeconds(delaySecs + math.random())
end

function GetSpawnDelay()
    if(this:HasObjVar("SpawnerProfile")) then
    	local spawnerProfile = this:GetObjVar("SpawnerProfile")
    	return SpawnerProfile[spawnerProfile].Frequency
    end	
    local delaySecs = this:GetObjVar("spawnDelay") or DEFAULT_DELAY_SECS
    return TimeSpan.FromSeconds(delaySecs + math.random())
end

function GetSpawnLoc()    
    -- local spawnRegion = nil
    -- if(this:HasObjVar("spawnRegions")) then
    --     -- DAB TODO: Weight based on size of region
    --     local spawnRegions = this:GetObjVar("spawnRegions")
    --     spawnRegion = spawnRegions[math.random(1,#spawnRegions)]
    -- else
    --     spawnRegion = this:GetObjVar("spawnRegion")
    -- end

    -- if(spawnRegion) then
    --     if(isDungeon) then
    --         return GetRandomDungeonSpawnLocation(spawnRegion)
    --     else
    --         return GetRandomPassableLocation(spawnRegion,true)
    --     end
    -- end

    if(this:HasObjVar("SpawnerProfile")) then
    	local spawnerProfile = this:GetObjVar("SpawnerProfile")
    	if(SpawnerProfile[spawnerProfile].SpawnRadius) then
            return GetRandomPassableLocationInRadius(this:GetLoc(),spawnRadius,true)
    	end
    end	    

    -- local spawnRadius = this:GetObjVar("spawnRadius")
    -- if(spawnRadius) then
    --     if(isDungeon) then
    --         return GetRandomDungeonSpawnLocationInRange(this:GetLoc(), spawnRadius)
    --     else
    --         return GetRandomPassableLocationInRadius(this:GetLoc(),spawnRadius,true)
    --     end
        
    -- end    

    return this:GetLoc()
end

function ShouldSpawn(spawnData, spawnIndex, templateId)
    if(mIsPendingCreate) then
        return false
    end

    local isValid = spawnData[spawnIndex].ObjRef and spawnData[spawnIndex].ObjRef:IsValid()
    local isDead = isValid and spawnData[spawnIndex].ObjRef and spawnData[spawnIndex].ObjRef:IsMobile() and IsDead(spawnData[spawnIndex].ObjRef)

    -- the mob is still on the map and hes not a pet, no spawn
    if( isValid and not(isDead) and not(IsPet(spawnData[spawnIndex].ObjRef)) ) then
        return false
    end

    -- If a spawn already exists at our spot we need to not spawn at all!
    if( FoundSpawn(templateId) ) then
        return false
    end

    local spawnDelay = GetSpawnDelay(spawnInfo)
    local spawnSpeedModifier = HelperSpawns.CalculateSpawnSpeedModifier(this, mSpawnPower) or mDefaultSpawnSpeedModifier
    --DebugMessage("spawnSpeedModifier [simple_mob_spawner] " .. tostring(spawnSpeedModifier))
    spawnDelay = TimeSpan.FromSeconds( spawnDelay.TotalSeconds * spawnSpeedModifier )
    --DebugMessage(this,"Checking spawnDelay [simple_mob_spawner] "..tostring(spawnDelay.TotalSeconds))
    if(spawnDelay.TotalSeconds ~= 0) then
        -- this mob just disappeared or died
        local goneTime = spawnData[spawnIndex].GoneTime
        if( spawnData[spawnIndex].ObjRef and not(goneTime) ) then  
            --DebugMessage(this,"MOB DEAD OR DESPAWNED")   
            spawnData[spawnIndex].GoneTime = DateTime.UtcNow
            return false
        end

        -- are we waiting on the spawn timer
        if(goneTime and DateTime.UtcNow < (goneTime + spawnDelay)) then
            --DebugMessage(this,"Waiting on spawn timer")   
            return false
        end
    end

    -- roll to spawn
    local spawnChance = this:GetObjVar("spawnChance") or DEFAULT_SPAWN_CHANCE

    if(this:HasObjVar("SpawnerProfile")) then
    	local spawnerProfile = this:GetObjVar("SpawnerProfile")
    	spawnChance = SpawnerProfile[spawnerProfile].SpawnChance
    end	

    if(spawnChance < 1 and math.random() > spawnChance) then   
        spawnData[spawnIndex].GoneTime = DateTime.UtcNow
        return false
    end

    return true
end

function CheckSpawn()
    -- add some randomness so there arent all spawning at the same time
    this:ScheduleTimerDelay(GetSpawnPulse(), "spawnTimer")

    if (this:HasObjVar("Disable")) then return end

    if( HelperSpawns.IsDynamicProximitySpawner(this) ) then
        -- If we don't have the spawn power already we need to calculate it
        if( mSpawnPower == nil ) then
            --DebugMessage("Calculating Spawn Power")
            mSpawnPower = HelperSpawns.CalculateFullSpawnPower(this)
        end
    end
    

    local spawnData = this:GetObjVar("spawnData")        
    local spawnCount = this:GetObjVar("spawnCount") or DEFAULT_SPAWN_COUNT

    if(this:HasObjVar("SpawnerProfile")) then
    	local spawnerProfile = this:GetObjVar("SpawnerProfile")
    	spawnCount = SpawnerProfile[spawnerProfile].SpawnCount
    end	

    local templateId = nil
    local spawnTable = this:GetObjVar("spawnTable")
    if(spawnTable) then
        local totalWeight = 0
        for i,spawnEntry in pairs(spawnTable) do
            totalWeight = totalWeight + (spawnEntry.Weight or 1)
        end

        local weightRoll = math.random() * totalWeight
        totalWeight = 0
        for i,spawnEntry in pairs(spawnTable) do
            if( weightRoll <= totalWeight + spawnEntry.Weight ) then
                templateId = spawnEntry.Template
                break
            end
            totalWeight = totalWeight + (spawnEntry.Weight or 1)
        end        
    end

    -- CHECK OUR SPAWNER PROFILES

    if(this:HasObjVar("SpawnerProfile")) then
    	local spawnerProfile = this:GetObjVar("SpawnerProfile")
    	templateId = SpawnerProfile[spawnerProfile].Templates[math.random(1,#SpawnerProfile[spawnerProfile].Templates)]
        if(SpawnerProfile[spawnerProfile].RareSpawnChance) then
            if(math.random() <= SpawnerProfile[spawnerProfile].RareSpawnChance) then
                templateId = SpawnerProfile[spawnerProfile].RareSpawnTemplates[math.random(1,#SpawnerProfile[spawnerProfile].RareSpawnTemplates)]
            end
        end
    end

    if not(templateId) then
        templateId = this:GetObjVar("spawnTemplate")
    end

    if( templateId == nil ) then
        --DebugMessage("ERROR: Simple mob spawner picked invalid template: "..tostring(this.Id).. " SpawnerTemplate: "..tostring(this:GetCreationTemplateId()))
        return
    end
    
    -- if (spawnData ~= nil) then
    --     if (this:HasObjVar("NightSpawn")) then
    --         if (IsDayTime()) then
    --             local playerAround = not (#FindObjects(SearchPlayerInRange(20)) == 0)
    --             if not playerAround then
    --                 for i=1,#spawnData do
    --                     if (spawnData[i].ObjRef ~= nil and spawnData[i].ObjRef:IsValid()) then
    --                         spawnData[i].ObjRef:Destroy()
    --                         spawnData[i].GoneTime = DateTime.UtcNow
    --                     end
    --                 end
    --             end
    --             return
    --         end
    --     elseif (this:HasObjVar("DaySpawn")) then
    --         if (IsNightTime()) then 
    --             local playerAround = not (#FindObjects(SearchPlayerInRange(20)) == 0)
    --             if not playerAround then
    --                 for i=1,#spawnData do
    --                     if (spawnData[i].ObjRef ~= nil and spawnData[i].ObjRef:IsValid()) then
    --                         spawnData[i].ObjRef:Destroy()
    --                         spawnData[i].GoneTime = DateTime.UtcNow
    --                     end
    --                 end
    --             end
    --             return
    --         end
    --     end
    -- end
    -- NOTE: If spawn count is made smaller, we are not destroying the extra mobs 
    if spawnData == nil then
        spawnData = {}
    elseif #spawnData > spawnCount then
    	for i=spawnCount+1,#spawnData do
            table.remove(spawnData,i)
        end
    end        

    --DebugMessage("---CheckSpawn SpawnCount: "..spawnCount.." spawnChance: "..tostring(spawnChance))     

        --DebugMessage("SPAWNING")
        for i=1, spawnCount do
            --DebugMessage("---Checking i="..i)     
        if(spawnData[i] == nil) then spawnData[i] = {} end
        if( ShouldSpawn(spawnData, i, templateId) ) then   
                --DebugMessage("---Create "..templateId)     
                local spawnLoc = GetSpawnLoc()
                if(spawnLoc) then
                    mIsPendingCreate = true
                	CreateObj(templateId, spawnLoc, "mobSpawned", i)                    
                    -- DAB NOTE: SHOULD WE ONLY SPAWN ONE PER PULSE? AND SHOULD WE ONLY ROLL ONCE?
                	break
                end
            end
        end

    this:SetObjVar("spawnData", spawnData)
end

RegisterEventHandler(EventType.Message,"Activate",function ( ... )
    this:DelObjVar("Disable")
end)
RegisterEventHandler(EventType.Message,"Deactivate",function ( ... )
    this:SetObjVar("Disable",true)
end)
RegisterEventHandler(EventType.Message,"RemoveSpawnedObject",function (targetObj)
    local spawnData = this:GetObjVar("spawnData")

    for i,spawnInfo in pairs(spawnData) do
        if(spawnInfo.ObjRef == targetObj) then
            spawnInfo.ObjRef = nil
            spawnInfo.GoneTime = DateTime.UtcNow
        end
    end
    this:SetObjVar("spawnData",spawnData)
end)

RegisterSingleEventHandler(EventType.ModuleAttached, "simple_mob_spawner", 
	function()
        if(initializer and initializer.SpawnTable) then
            this:SetObjVar("spawnTable",initializer.SpawnTable)
        end
        if(initializer and initializer.SpawnObjVars) then
            this:SetObjVar("spawnObjVars",initializer.SpawnObjVars)
        end

        CheckSpawn()
		
	end)

function FoundSpawn( templateId ) 
    nearbyObjects = FindObjects(SearchTemplate(templateId,1))
    if (#nearbyObjects > 0) then
        for i=1, #nearbyObjects do 
            local foundObj = nearbyObjects[i]
            if( foundObj ~= objref ) then
                return true
            end
        end
    end
    return false
end


function MobSpawned(success, objref, index)
    mIsPendingCreate = false
    
    if( success) then

        -- If we find something that has spawned right beside us we want to delete it
        CallFunctionDelayed(TimeSpan.FromSeconds(math.random( 1,10 )),function()
            if( FoundSpawn( objref:GetCreationTemplateId() ) ) then
                --objref:Destroy()
            end
        end)

        local spawnData = this:GetObjVar("spawnData") 
        if(spawnData[index]) then
            spawnData[index].ObjRef = objref
            spawnData[index].GoneTime = nil
        else
            spawnData[index] = { ObjRef = objref }
        end            
        this:SetObjVar("spawnData",spawnData)
        objref:SetObjVar("Spawner",this)
        --DebugMessage("Set facing to " .. tostring(this:GetFacing()))
        if(objref:IsMobile()) then
            if(this:HasObjVar("spawnRadius")) then
                objref:SetFacing(math.random() * 360)
            else
                objref:SetFacing(this:GetFacing())
            end
        end

        local spawnObjVars = this:GetObjVar("spawnObjVars")
        if(spawnObjVars) then
            for varName,varData in pairs(spawnObjVars) do
                objref:SetObjVar(varName,varData)
            end
        end

        local spawnPath = this:GetObjVar("spawnPath")
        if( spawnPath ) then
            objref:SetObjVar("MyPath",spawnPath)
        end

        local spawnerProfile = this:GetObjVar("SpawnerProfile")
        if(SpawnerProfile[spawnerProfile].LootTables) then
            local finalLootTable = {}
            for index,lootTableValue in pairs(SpawnerProfile[spawnerProfile].LootTables) do
                if(SpawnerProfile[spawnerProfile].LootTables[objref:GetCreationTemplateId()]) then
                    for indexTemplate, templateValue in pairs(SpawnerProfile[spawnerProfile].LootTables[objref:GetCreationTemplateId()]) do
                        table.insert(finalLootTable,templateValue)
                    end
                    objref:SetObjVar("LootTables",finalLootTable)
                    return
                end
            end
        end

    end
end

-- this allows the function to be overridden in a mod
RegisterEventHandler(EventType.CreatedObject, "mobSpawned", function(...) MobSpawned(...) end)

RegisterEventHandler(EventType.Timer, "spawnTimer", 
	function()
		CheckSpawn()
	end)

this:ScheduleTimerDelay(GetSpawnPulse(), "spawnTimer")


------------------------------------------------------------------------------
-- EDITED THE SIMPLE_MOB_SPAWNER TO BE A PROFILE SPAWNER
------------------------------------------------------------------------------

RegisterEventHandler(EventType.ModuleAttached, GetCurrentModule(),
function()  
    AddUseCase(this,"[FF7700]World Building",false,"IsGod")        
    AddUseCase(this,"[FFFF00]Duplicate",false,"IsGod")
    AddUseCase(this,"[FFFF00]Face Me",false,"IsGod")
    AddUseCase(this,"[FFFF00]Move",false,"IsGod")
    AddUseCase(this,"[FFFF00]SeedClip",false,"IsGod")
    AddUseCase(this,"[FF7700]Spawner Functions",false,"IsGod")        
    AddUseCase(this,"[FFFF00]Reset",false,"IsGod")
    AddUseCase(this,"[FFFF00]Select Profile",true,"IsGod")  

    -- delay to allow time for the profile to attached as an objvar by the seedobjects generation
    CallFunctionDelayed(TimeSpan.FromSeconds(5),function()
        UpdateTooltip()
    end)      
end)

RegisterEventHandler(EventType.Message, "UseObject", 
function(user,usedType)

    if(usedType == "[FFFF00]Duplicate") then
        user:RequestClientTargetLoc(this,"duplicateAtTarget")
        user:SystemMessage("select a location you wish to place a copy of this spawner at")
    end
    if(usedType == "[FFFF00]Face Me") then
        user:SetFacing(user:GetLoc():YAngleTo(this:GetLoc()))
        local newRot = Loc(0,user:GetFacing()-180,0)
        this:SetRotation(newRot)            
    end 
    if(usedType=="[FFFF00]SeedClip") then
        HandleClip(user)
    end
    if(usedType == "[FFFF00]Move") then
        user:RequestClientTargetLoc(this,"moveToTarget")
        user:SystemMessage("select a location to move this spawner too")
    end 
    if(usedType == "[FFFF00]Reset") then
        if(this:HasObjVar("spawnData")) then
            for index,spawnedObject in pairs(this:GetObjVar("spawnData")) do
                spawnedObject.ObjRef:Destroy()
            end
            this:DelObjVar("spawnData") 
        end
    end      

    if(usedType=="[FFFF00]Select Profile") then
        ShowProfileSelectionWindow(user,category)
    end

end)

RegisterEventHandler(EventType.ClientTargetLocResponse,"duplicateAtTarget",
function(success,location)
    if(success and location ~= nil) then
        CreateObj("profile_spawner",location,"duplicateCreated")
    end
end)

RegisterEventHandler(EventType.CreatedObject, "duplicateCreated", 
function(success, objref, index)
    if(this:HasObjVar("SpawnerProfile")) then
        if( success) then
            CallFunctionDelayed(TimeSpan.FromSeconds(1),function ( ... )
                objref:SetObjVar("SpawnerProfile",this:GetObjVar("SpawnerProfile"))
            end)
        end
    end
    if(this:HasObjVar("NoReset")) then
        objref:SetObjVar("NoReset",true)
    end
    if(this:HasObjVar("IsSeedObject")) then
        objref:SetObjVar("IsSeedObject",true)
    end    
end)

RegisterEventHandler(EventType.ClientTargetLocResponse,"moveToTarget",
function(success,location)
    if(success and location ~= nil) then
        this:PlayEffect("TeleportToEffect")
        this:SetWorldPosition(location)
        this:PlayEffect("TeleportFromEffect")
    end
end)

function HandleClip(user)
    if(this==nil) then return end
    if(this:GetObjVar("SpawnerProfile") == nil) then
        user:SystemMessage("[AA4444]Failed to Clip, no profile given")
    end
    local clipString = ""
    local finalString = ""
    local template = this:GetCreationTemplateId()
    local location = this:GetLoc()
    local rotation = this:GetRotation()
    local scale = this:GetScale()
    local dynamicObjectElement = xml.new("DynamicObject")
    local creationParams = template .. " " .. 
                           location.X .. " " ..
                           location.Y .. " " ..
                           location.Z .. " " ..
                           rotation.X .. " " ..
                           rotation.Y .. " " ..
                           rotation.Z .. " " ..
                           scale.X .. " " ..
                           scale.Y .. " " ..
                           scale.Z
    -- add the objectcreationparams element and element data
    dynamicObjectElement:append("ObjectCreationParams")[1] = creationParams

    local objVarOverridesElement = xml.new("ObjVarOverrides")
    local spawnTemplateElement = xml.new("StringVariable")
    -- spawnTemplate
    spawnTemplateElement[1] = this:GetObjVar("SpawnerProfile")
    spawnTemplateElement["Name"] = "SpawnerProfile"
    objVarOverridesElement:append(spawnTemplateElement)

    dynamicObjectElement:append(objVarOverridesElement)   

    clipString = string.gsub(tostring(dynamicObjectElement),"&quot;","\"")
    finalString = AddIndentX2(clipString)
    io.popen('clip','w'):write(finalString):close()
    user:SystemMessage("Object Cliped CTRL+V To Paste")
end

function AddIndentX2(str)
  local result = {}
  local finalResult = ""
  for line in str:gmatch '[^\n]+' do
    table.insert(result, line)
  end
  for k,v in pairs(result) do
    finalResult = finalResult .. "\t\t"..v.."\n"
  end
  return finalResult
end

RegisterEventHandler(EventType.DynamicWindowResponse,"WorldBuilderDynamicWindow", 
function (user,buttonId,textField)
    local rawButtonId = StringSplit(buttonId, ":")
    local button = rawButtonId[1]
    local args = rawButtonId[2]

    if(button=="ProfileKey") then
        if(args ~= nil) then
            this:SetObjVar("SpawnerProfile",args)
            if(this:HasObjVar("spawnData")) then
                for index,spawnedObject in pairs(this:GetObjVar("spawnData")) do
                    spawnedObject.ObjRef:Destroy()
                end
                this:DelObjVar("spawnData") 
            end
            ShowProfileSelectionWindow(user)
            UpdateTooltip()            
        end
    end
end)

function ShowProfileSelectionWindow(user)
    local dynamicWindow = DynamicWindow("WorldBuilderDynamicWindow","",300,450,-300,-50,"TransparentDraggable","Center")
    local scrollElement = nil
    local profileName =  " - Non Selected -"
    -- if(this:HasObjVar("SpawnerProfile") ~= nil) then
    --     profileName = SpawnerProfile[this:GetObjVar("SpawnerProfile")].ProfileName
    -- else
    --     profileName = "- None Selected -" 
    -- end

    if(this:HasObjVar("SpawnerProfile")) then
        profileName = SpawnerProfile[this:GetObjVar("SpawnerProfile")].ProfileName
    end
    --Create Background
    dynamicWindow:AddImage(1,1,"HueSwatch",300,450,"Simple","000000")
    -- header background
    dynamicWindow:AddImage(1,1,"HueSwatch",300,24,"Simple","450000")    
    -- header title
    dynamicWindow:AddLabel(150,3,"[959555][Profile Selector]",280,24,24,"center")
    dynamicWindow:AddLabel(150,32,"[959555]select a profile below for this spawner",280,16,16,"center")
    dynamicWindow:AddLabel(150,52,"[959555]".."[CCCC88]"..profileName,300,20,20,"center")    
    local scrollWindow = ScrollWindow(10,95,270,340,20)
    dynamicWindow:AddScrollWindow(scrollWindow)    
    local switchcolors = true
    local color = "350000"
    
    local profileNames = {}
    for k,v in pairs(SpawnerProfile) do
        table.insert(profileNames,{ProfileName=v.ProfileName,ProfileKey=k})
    end

    table.sort(profileNames,
    function(a,b)
        return a.ProfileName < b.ProfileName 
    end)

    for profileKey,profileData in pairs(profileNames) do
        scrollElement = ScrollElement()
        if(switchcolors== false) then
            color="452525"
        else
            color="350000"
        end
        scrollElement:AddImage(1,1,"HueSwatch",250,20,"Simple",color)
   
        scrollElement:AddLabel(6,3,"[959555]"..SpawnerProfile[profileData.ProfileKey].ProfileName,300,18,18,"left")
        scrollElement:AddButton(1,1,"ProfileKey:"..profileData.ProfileKey,"",220,20,SpawnerProfile[profileData.ProfileKey].ProfileTooltip,"",false,"Invisible")    
        scrollWindow:Add(scrollElement)
        switchcolors = not switchcolors
    end

    dynamicWindow:AddScrollWindow(scrollWindow)

    DrawCloseButton(281,3,dynamicWindow)
    user:OpenDynamicWindow(dynamicWindow)
end

function DrawCloseButton(x,y,dynamicWindow)
    dynamicWindow:AddButton(x,y,"CloseButton","",18,18,"[CCCC00]Close Button\n[959555]clicking this will close the profile spawner dynamic window.","",true,"Invisible")
    --draw the "X" string
    dynamicWindow:AddLabel(x+5,y+3,"[959555]X",18,18,18,"left")
end


function UpdateTooltip()
    if(this:HasObjVar("SpawnerProfile")) then
        local profileSpawnerData = SpawnerProfile[this:GetObjVar("SpawnerProfile")]
        
        SetTooltipEntry(this,"ProfileAttributesHeader","[CCCC55]Profile Details",1000)        
        SetTooltipEntry(this,"ProfileName","[FF7700]"..profileSpawnerData.ProfileName,990)
        SetTooltipEntry(this,"ProfileTooltip","[DDDDDD]"..profileSpawnerData.ProfileTooltip,980)
    else
        SetTooltipEntry(this,"ProfileAttributesHeader","[CCCC55]-- NO PROFILE SELECTED --",1000) 
    end
end

RegisterEventHandler(EventType.LoadedFromBackup, "", function()
    UpdateTooltip()
end)