-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- Chain Explosion when casted on your target will deal damage over time on each pulse tick.
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.ChainExplosion = 
{
	-------------------------------------------------------------------------------------------
	-- Core Variables
	-------------------------------------------------------------------------------------------
	-- Targets
	Caster = nil,
	Target = nil,
	-- Internal Variables
	PulseFrequency = 2,
	PulseMax = 2,
	CurrentPulse = 0,
	MinDamage = 10,
	MaxDamage = 16,
	-- % of the total max life, mana or health regained back
	LeechEfficacy =
	{
		Health = 0.035,
		Mana = 0.025,
		Stamina = 0.03,
	},
	-------------------------------------------------------------------------------------------
	-- Core Mobile Effect Functions/Overrides
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		self.Target = target
		self.Caster = self.ParentObj
		-- if a player is casting, do they get extra pulses added to the spell effect?
		if(self.Caster:IsPlayer()) then
			local extraChain = GetCachedMobileModByName("ChainExplosionSpecialExtraExplosion",self.Caster) or 0
			self.PulseMax = self.PulseMax + extraChain
			-- since its a player, lets notify them something bad is about to happen
			self.Target:SystemMessage("[FF7700]you feel something burning and building inside...")
		end
	end,
	OnExitState = function(self,root)
		if(self.Caster:IsPlayer()) then
			self.Caster:SystemMessage("[FF7700]your chain explosion expired on ".. self.Target:GetName())
		end
	end,
	OnStack = function(self,root,target,args)
		-- if someone casts chain explosion again on the target, reset the pulse count to zero 
		self.CurrentPulse = 0
	end,
	GetPulseFrequency = function(self,root)
		if(self.Caster:IsPlayer()) then
			local frequencyReduction = GetCachedMobileModByName("ChainExplosionSpecialShortFuse",self.Caster) or 0
			if(frequencyReduction > 0) then
				self.PulseFrequency = self.PulseFrequency - frequencyReduction
				if(self.PulseFrequency < 0.5) then self.PulseFrequency = 0.5 end
			end
		end
		return TimeSpan.FromSeconds(self.PulseFrequency)
	end,
	AiPulse = function(self,root)
		self.CurrentPulse = self.CurrentPulse + 1
		if ( IsDead(self.Caster) or self.CurrentPulse > self.PulseMax ) then
			EndMobileEffect(root)
		else
			self.DoTickEffect(self,root)
		end		
	end,
	-------------------------------------------------------------------------------------------
	-- Custom Functions
	-------------------------------------------------------------------------------------------
	DoTickEffect = function (self,root)
		if(IsDead(self.Target)) then EndMobileEffect(root) end
		local damage = math.random(self.MinDamage,self.MaxDamage)
		if(self.Caster:IsPlayer()) then
			damage = damage + (GetCachedMobileModByName("DamageBonusSpellChainExplosion",self.Caster) or 0)			
			local flameJumpChance = (GetCachedMobileModByName("DamageBonusSpellChainExplosion",self.Caster) or 0)
		end
		self.Target:PlayEffectWithArgs("FireballExplosionEffect",2,"Bone=Spine")
		self.Target:PlayObjectSound("event:/magic/fire/magic_fire_fireball_impact",false)
		self.Target:SendMessage("DamageInflicted",self.Caster,damage,"MAGIC",false,false,false,self.Caster)
		-- check for leeching life, stamina and mana
		self.CheckLeechEffects(self,root)
	end,
	CheckLeechEffects = function(self,root)
		local lifeLeech = GetCachedMobileModByName("OnHitSpellChainExplosionLeechHealth",self.Caster) or 0
		local manaLeech = GetCachedMobileModByName("OnHitSpellChainExplosionLeechMana",self.Caster) or 0
		local staminaLeech = GetCachedMobileModByName("OnHitSpellChainExplosionLeechStamina",self.Caster) or 0
		-- leech is a % based mod as long as its higher then 0 and it makes the % chance roll, activate the leeching
		if( lifeLeech >= 1 and math.random(100) <= lifeLeech) then
			local maxHealth = GetMaxHealth(self.Caster)
			local curHealth = GetCurHealth(self.Caster)
			local healAmount = math.ceil(maxHealth * self.LeechEfficacy.Health) or 1
			SetCurHealth(self.Caster,curHealth + healAmount)
		end
		if( manaLeech >= 1 and math.random(100) <= manaLeech) then
			local maxMana = GetMaxMana(self.Caster)
			local curMana = GetCurMana(self.Caster)
			local manaAmount = math.ceil(maxMana * self.LeechEfficacy.Mana) or 1
			SetCurMana(self.Caster,curMana + manaAmount)
		end
		if( staminaLeech >= 1 and math.random(100) <= staminaLeech) then
			local maxStamina = GetMaxStamina(self.Caster)
			local curStamina = GetCurStamina(self.Caster)
			local staminaAmount = math.ceil(maxStamina * self.LeechEfficacy.Stamina) or 1
			SetCurStamina(self.Caster,curStamina + staminaAmount)
		end
	end
}