-----------------------------------------------------------------------------------------------
-- VERSION 1.0
-----------------------------------------------------------------------------------------------
-- Duration Scaling
-- * base duration is 10 seconds
-- * bonuse duration 1 - 20 seconds based on magic focus enhancment
-- Defense Bonus Scaling
-- * base defense value 10%
-- * bonus defense is 0%-25% more defense based on magic focus enhancment
-----------------------------------------------------------------------------------------------
MobileEffectLibrary.DivineAegis = 
{
	-------------------------------------------------------------------------------------------
	-- CORE VARIABLES
	-------------------------------------------------------------------------------------------
	BaseBonusDefense = 0.10,
	Duration = SpellData.SpellEffectDuration.BuffVeryShort,
	-------------------------------------------------------------------------------------------
	-- CORE FUNCTIONS
	-------------------------------------------------------------------------------------------
	OnEnterState = function(self,root,target,args)
		local enhancementSkill = GetSkillLevel(target,"MagicFocusEnhancement") 
		local bonusEnhancementDuration = (enhancementSkill / 100) * 20
		local bonusEnhancementDefense = (enhancementSkill/400)
		local bonusDefense = self.BaseBonusDefense + bonusEnhancementDefense
		self.Duration = self.Duration + TimeSpan.FromSeconds(bonusEnhancementDuration)		
		-- apply buff icons
		if ( self.ParentObj:IsPlayer() ) then
			AddBuffIcon(self.ParentObj, "DivineAegis", "Divine Aegis", "Block", "May the light protect you when you need it most.")
		end
		-- set mobile mods
		SetMobileMod(self.ParentObj, "DefenseTimes", "DivineAegis", bonusDefense)
		-- play spell effects
		self.PlayEffects(self)
	end,
	OnExitState = function(self,root)
		-- remove buff icons and mobile modes
		if ( self.ParentObj:IsPlayer() ) then
			RemoveBuffIcon(self.ParentObj, "DivineAegis")
			self.ParentObj:SystemMessage("[FFFF77]The light of divine aegis leaves you...")
		end
		SetMobileMod(self.ParentObj, "DefenseTimes", "DivineAegis", nil)
	end,
	OnStack = function(self,root,target,args)
		-- refresh duration
		root.ParentObj:ScheduleTimerDelay(self.Duration, root.PulseId.."-"..root.CurStateName)
		self.PlayEffects(self)
	end,
	GetPulseFrequency = function(self,root)
		return self.Duration
	end,
	AiPulse = function(self,root)
		EndMobileEffect(root)
	end,
	-------------------------------------------------------------------------------------------
	-- CUSTOM FUNCTIONS
	-------------------------------------------------------------------------------------------
	PlayEffects = function(self,root,target,args)
		self.ParentObj:PlayObjectSound("event:/magic/misc/magic_water_restoration",false)
		self.ParentObj:PlayEffectWithArgs("HolyEffect",0,"Bone=Ground")
		self.ParentObj:PlayEffectWithArgs("YellowPortalEffect",2.5,"Bone=Ground")
		self.ParentObj:SystemMessage("[FFFF77]As you embrace the devine fire within, you feel unstoppable...")	
	end,
}
